#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
/******************************************************************
*
* Author: James A. Green,  First Edition: March 22, 1992. Quick C.
*     with references	      Modified 1/26/97 for WINDOWS.
*     contributed personally           7/7/97  for upgrade.
*     by S. Chandrasekhar.             11/30/99 for Borland C++ Compiler.
*                                      12/13/99 for rotational flyapart
*                                               sqrt exception fix.
*                                      01/14/2000 for invalid-input-resistant
*                                                 replacement for simple scanf().
*                                      01/21/2000 - Uses 4.761176 for bolometric-corrected
*                                                   magnitude of the Sun M_bol_sun, and uses
*                                                   M_bol_sun in the calculation of both
*                                                   bolometricly corrected and uncorrected
*                                                   values.
*        
*		           Copyright: C  James A. Green, 1999.
*                          GREENWOOD RESEARCH BOOKS & SOFTWARE.
*                          Catalog: http:ray-green.tripod.com.
*                          Email Reports: greenwood@ksee24mail.com.
*
*                          USER-MODIFIABLE SOFTWARE - NOTICES.
* CHANDRA -                The customer may make a copy of CHANDRA's
* Chandrasekhar Sets       software, and modify it to explore stellar
* of White Dwarfs &        structure modeling according to his own plan.
* Supernova Cores          For instance, the customer may introduce
* with stellar structure   modified equations of state, graphics, ect. 
* software on diskette     Resale rights for the original listing, the
* for WINDOWS 95/98/NT.    original source code and the associated book,
*                          CHANDRA, are retained by James A. Green.                        
*
*                     BUGS in CHANDRA may be reported by email via
*                          jim_green@altavista.com or through
*                          greenwood@ksee24mail.com.
*                          Telephone: [316] 722-4954.
*                          FAX: [775] 206-1391. Or write to:
*                          James A. Green, Greenwood Research,
*                          1036 Murray Court,
*                          Wichita, Kansas 67212-4041.
*                          Replacement software may be provided or not, 
*                          depending on time- and resource- availability.
*                          CHANDRA seems to work fine on a COMPAC
*                          Presario 1230 laptop with a 233 Mhz Pentium CPU.
*                          BUGS introduced by the customer in the course
*                          of his own program of investigation with
*                          an auxilliary copy of CHANDRA are his
*                          responsibility. - Jim Green, Dec.1, 1999.
*                          At this time some equations of state for
*                          neutron stars convey suspicious results.
*                          Otherwise, a square root domain error
*                          sometimes occurs if a rotation is rapid
*                          enough to grossly distort a GR star so that
*                          it's ellipticity is large.
*
*  HOW TO RUN CHANDRA.EXE
*  To execute chandra.exe, you may use RUN from the START menu of WINDOWS 95/98,
*  directly from the diskette if desired by using RUN on a:chandra.exe in drive a.
*  Otherwise you may write chandra.exe into any Windows directory to install it.
*
*  CHANDRA produces the result file chandra.txt in the directory in which
*  chandra.exe resides.  This file contains basic results for each stellar model
*  specified by the user during the run. Usually you must read this file, not
*  just the printout on the screen, to get all required data. If desired, CHANDRA
*  produces a detailed structure file of stellar variables from the center to the
*  surface for each model for which such a listing is specified.  This file is
*  stellara.txt. Up to 100 stellar models may be processed in a single
*  invocation of CHANDRA.
*
*  The files chandra.txt and stellara.txt are automatically rewritten each time
*  chandra.exe is executed, so save result files under different names.
*
*  CHANDRA produces a restart file, restart.txt, that may be used to resume model
*  processing in the middle of doing a long list of models if the machine is 
*  powered down and subsequently powered back up again. Recovery proceeds by 
*  re-executing CHANDRA and selecting the restart option (2) from it's initial 
*  menu of options. The restart works after a reset or power-up sequence, so
*  that the desired model sequence is produced in it's entirety, with model
*  execution resuming at the beginning of the model that was interrupted.
*
*
*  CHANDRA - STELLAR STRUCTURE CODE FOR WINDOWS -
*            Compute a Chandrasekhar mass limit set,
*            with and without corrections to white
*            dwarfs and supernova cores due to general
*            relativity, electrostatic screening effects,
*            neutronization and rotation. Computes
*            white dwarfs, neutron stars, pulsars,
*            supernova cores, and quark stars.
*	     Also, examine neutron object mass limits for a
*	     heavy neutron imploder - using simple relativisitic
*	     neutron sea and D-quark sea models, screened
*	     and unscreened. Includes routines for sketching
*            a main sequence star from basic variables
*               r = distance in LY, m = visual magnitude,
*               Tsurface (a function of spectral type).
*
*  From an array of input starting densities and
*  conditions, integrate the equation of hydro-
*  static equilibrium from the center of the
*  star outward until the pressure is zero.
*  This yeilds a sphere of radius R and known
*  mass.  In the limit as the starting density
*  goes to infinity, we have the Chandrasekhar
*  limit for the mass, at which time the radius
*  goes to zero.  The program can be used to
*  compute a set of characteristics for the
*  Chandrasekhar set of white dwarfs or super-
*  nova cores, optionally including rotations.
*  In the rotations case, the program integrates
*  the pressure gradient on the minor axis,
*  the axis of rotation in this particular problem,
*  calculating the major axis also and the surfaces
*  of constant potential to find mass increments.
*  CHANDRA also calculates binding energies,
*  moments of inertia, total internal thermal
*  energy, gravitational redshifts, ellipticity
*  due to rotation, and other variables for its
*  model stars and star cores.
*
*  If white dwarf calculations are desired, the
*  program will query for Z, the average nuclear
*  charge, which must be an integer, and for A.
*  Z[C,Mg,Si,S,Fe] =  6,12,14,16,26.
*  A[C,Mg,Si,S,Fe] = 12,24,26,30,56.
*
*  For each starting density in the set of
*  objects, we can specify a correction by a
*  numeric condition code:
*
*  0 - no corrections - Newtonian gravity. Original, 1930's, Chandrasekhar.
*  1 - general relativity only.  Original with general relativity, Kaplan.
*  2 - newtonian gravity, Eqn Of State shieldg. ala E.E. Salpeter 1960's.
*  3 - general relativity, Eqn Of State shieldg. ala E.E. Salpeter 1960's.
*  4 - newtonian gravity, neutronization only.
*  5 - general relativity, neutronization only. J. Robert Oppenheimer, 40's.
*  6 - newtonian gravity, neutronization & Equation Of State (EOS).
*  7 - general relativity, neutronization & Equation Of State.
*  8 - general relativity, pure Fermi neutron gas. ala J. Robert Oppenheimer.
*  9 - Harrison-Wheeler EOS (linear approx.) Newtpnian gravity.
* 10 - Harrison-Wheeler EOS (lin.ap.) general rel. ala John Archibald Wheeler.
* 11 - Harrison-Wheeler EOS (l.a.),gr,elec.shieldg. ala John Archibald Wheeler.
* 12 - Baym-Bethe EOS. ala Hans Bethe.
* 13 - Bowers & Deeming non-relativistic degenerate neutron gas. (Approx.)
* 14 - Bowers & Deeming non-rel. degen. neut. gas. with Zeldovich
*       quark P = (ro*c**2)/3 realm beyond ro = 10**16 gr/cm**3.
* 15 - Neutron star as D-quark sea, electrostatic screening neglected.
* 16 - Neutron star as U-quark screened D-quark sea.
* 17 - Neutron star as a Fermi gas of heavy point neutrons.
*
*  All equations of state may terminate optionally
*  in the Zeldovich quark-model EOS, P = ro*[c**2]/3.
*
*  Rotation is a special option that is specified
*  independently of the other options. The program
*  queries for the angular velocity for each
*  rotating model. Uniform rotation assumed.
*  The program might be modified for differential
*  rotation using a series for the angular velocity.
*  Rotations may be done slowly but accurately with
*  Green's functions or approximately by using a
*  spherically symmetric field modified by a
*  centrifugal potential.
*
*  Optionally, the stellar structure associated
*  with each density may be computed and written
*  to a disk file. The compute step delta_r is
*  automatically adaptive. However, it can be
*  divided by any integer for greater precision.
*  The output includes M(r), density(r), P(r).
*
*
*  PROGRAM RESTART and INTERRUPTION with Control-C:
*   Control-C interrupts do not work in WINDOWS,
*   although control-C does interrupt under MSDOS.
*   Some laptops require power-down to interrupt
*   CHANDRA, other systems have a reset button
*   that may be used.
*
*  If the program is interrupted after computing at
*  least one stellar model from its internal table
*  of starting model conditions, it saves all setup
*  and summary information to that point in CHANDRA.TXT.
*  Complete internal program state information is
*  saved in RESTART.TXT, automatically.
*  CHANDRA.TXT is rewritten after each model is
*  computed so that the whole process may be interrupted
*  without loosing results already computed. Restart
*  with this version of CHANDRA requires
*  requesting a restart when the program queries
*  for the star type [or restart] at the beginning
*  of the run.  If a restart is requested, the program
*  reinitializes its internal state from RESTART.TXT
*  and continues the program operation where it left off.
*
*
*  STRUCTURAL TABLE OF CONTENTS
*   Constants
*   Input Variables
*   Process Variables
*   Output Variables
*   Program Start - Main
*      Preface - Output White Dwarf, Pulsar Data.
*      Test Probes Section - Conditionally activated tests.
*         Test inverse sinh.
*         Test cube root.
*         Test - compare equation of state integral for electron gas.
*         Test equations of state and inverse equation of state.
*      Variable setup.
*      Interactive operator entry of stellar model variables.
*        Optional branch to polytrope() polytropic star quick sketch & exit.
*      Call routine for stellar model cal, build STELLARA.TXT struc file.
*      Save CHANDRA.TXT summary report file.
*      Exit.
*    file_list() - Display and store CHANDRA.TXT summary report file.
*    setdef()    - Preset variable arrays for stellar models.
*    showconditioncodes() - Printout condition code meanings.
*    changedefaults() - Allow user to spec step size, max # steps,
*                       optional quark equation of state at superhigh density.
*    processlist() - Step through list of model variables, cal models.
*      process() - Process mth list entry with starcal().
*        starcal() - Process mth list entry according to condition code.
*           newtonian() - Integrate from center to surface for mth model.
*                         Calculate mass, radius, pressure, density, ect.
*           genrel() - Integrate the Tolman-Oppenheimer-Volkoff equation of
*                      hydrostatic equilibrium from the center to the
*                      surface, using condition-code specified eqn of state.
*               inveos() - Inverse equation of state via binary search.
*               eos()    - Equation of state, controlled by condition code.
*                  bowers() - Electrostatic pressure correction according
*                             to Bowers and Deeming.
*                  f(x) - Integral form of relativisitic electron gas func.
*                  gg(y) - Kernel of integral representation of eqn of state.
*               eoshw() - Harrison-Wheeler eqn of state, lin. log-log. approx.
*               thermen() - Thermal energy per unit volume function.
*                   chi() - Supports energy per unit volume for elec. gas.
*               eosbb() - Baym-Bethe linear log-log model for eqn. of state.
*               nthroot()      - Calculate nth root.
*               cuberoot()     - Calculate the cube root.
*               seventhroot()  - Calculate the seventh root.
*               f2() - Analytic form of f() for elec. gas eqn. of state.
*               invsinh() - Inverse sinh() via binary search.
*               set_deltar() - Adaptively set the radial step size and
*                              conditionally scale it for the integration.
*    save() - Save machine state for restart after control-C interrupt.
*    rego() - Restore machine state from RESTART.TXT option, then exec.
*
* STAR SKETCH - Standard model stars, Main Sequence.
*
*    polytrope() - Polytropic star quick sketch branch.
*                  BASIC SECTION
*                  Computes L/Lsun, M/Msun, R/Rsun, absolute mag. M
*                  from m, spectral type or Tsurface, and the distance
*                  or parallax. Works with or without bolometric corrections.
*                  POLYTROPIC MODEL SECTION.
*                  Additionally, computes central values for the
*                  Temperature, Pressure, Radiation Pressure,
*                  Gas pressure, and density from
*                  M/Msun, L/Lsun, and the chemistry of the star,
*                  which may be specified via X,Y,Z abundances or
*                  via the molecular weight. 
*       findbeta - Solve (1 - b)/b**4  = C for b in Eddington standard model.
*       distancecal() - cal distance from parallax.
*       luminosity() - cal L/Lsun, M/Msun, R/Rsun, absolute mag. M,
*                      from r in LY, m, and Tsurface, including the
*                      bolometric correction BC in all quantities.
*       luminosity2() - cal L/Lsun, M/Msun, R/Rsun, absolute mag. M,
*                      from r in LY, m, and Tsurface, not including
*                      the bolometric correction for atmospheric absorption.
*       stellarmass() - M/Msun from L/Lsun.
*       stellarradius() - R/Rsun from L/Lsun and Tsurface.
*       tsurf() - Find Tsurface from spectral type tables.
*       listmainseq() - List typical main-sequence stars with measures.
*          sublist() - List the above stars w/o controls.
*
*   References
*
****************************************************************/
int bcflag = 0;              // COMPILE-TIME OPTION:   SET for option 0 = Zombeck.
//                              0->Zombeck bolometric corrections, 1990,
//                              1->Schwarzschild bolometric corrections,1950s.
double mu = 0.59435;         // default Schwarzschild molecular weight for
//                              Main Sequence stars in sketch routines.
int sketchflag;              // set to 1 while doing stellar quick sketches.
//
//   New Prototype definitions for Borland C++ Compiler, Nov.30, 1999
//
double cuberoot(double v);   /* cube root of v */
void bcprobe(double bc, int i); // test for non-negative B.C. and stop with position display.
double bolometric(double xt);   // Return the bolometric correction from the surface temp.
double eos(double v);        /* equation of state */
double bowers(double v);     /* bowers equation of state */
double f(double v);          /* EOS sub-function */
double f2(double v2);        /* EOS sub-function #2 */
double eoshw(double v);      /* Harrison-Wheeler EOS */
double eosbb(double v);      /* Bam-Bethe EOS  */
double gg(double v);         /* prototype for gg, Nov. 30, 1999 */
double chi(double v);        /* EOS sub-function */
double nthroot(double v);    /* nth root of v */
double seventhroot(double v); /* 7th root of v */
double invsinh(double v);     /*  inverse sinh(v). */
void basicvars(void);           /* compute L/Lsun,M/Msun, M, R/Rsun [Main Sequence] */
//                                 from  r in LY, m, and Tsurface
void luminosity2();           // cal L/Lsun, M/Msun, M, from r in pc and m.
double rsolar = 6.95987e10;  /* solar radius in cm */
double rsolkm = 6.95987e5;   /* solar radius in km */
double msolar = 1.9892e33;   /* solar mass in grams */
double lsolar = 3.862e33;    /* solar luminosity */
double hbar   = 1.055e-27;   /* Planck's constant in erg/sec */
double sigma  = 5.670e-5;    /* Stephan-Boltzman constant, cgs */
double c      = 2.9979e10;   /* light speed, cm/sec */
double kb     = 1.381e-16;   /* Boltzmann constant in ergs/degree */
double mproton = 1.673e-24;  /* proton mass in grams */
double mpion   = 2.49e-25;   /* pion mass in grams */
double alpha = 0.3333;	     /* default D-quark mass = alpha x neutron_mass */

int looplim = 2001;          /* default 2001 loops per model */
int tflag = 1;               /* test flag */
double tstx;                 /* test variable */
double den;                  /* dummy density variable */

/*                              INPUT VARIABLES                 */
double ro00;		     /* global current model starting density */
double density[100];         /* starting densities array         */
double alphax[100];	     /* D-quark mass = alpha x neutron mass */
double rotation[100];        /* rotation angular velocities      */
int condition[100];          /* condition codes 0-7              */
int calstruc[100];           /* cal codes. 1=cal structure,else 0. */
int dwarfz[100];             /* White dwarf Z for Wigner-Seitz delta P */
double dwarfa[100];          /* White dwarf A for Bowers/Deeming delta P */
int novas = 1;               /* if 1, use Z = 26 supernovae, else dwarfs */
int znuc = 26;               /* Z for electrostatic corrections  */
int m = 0;                   /* index to process-list variables  */
int j = 0;                   /* pointer to last list entry + 1   */
int jsave = 0;               /* save area for last list entry + 1 */
int gocode = 0;              /* terminate the entry list if = 1  */
int rotate = 0;              /* include rotations flag for whole set */
int greens = 0;              /* use Green's functions in rotation cal */
int quarks = 0;              /* include Zeldovich quark EOS for whole set */

int step_size = 5;           /* divide radial step by integer value */
double deltar = 0.0;         /* cal step size delta r */

int openflag = 0;            /* stellara.txt open when = 1 */
int defaultflag = 0;         /* switch for accepting deflts */
int defaultflg = 0;          /* 1 to proceed with deflts */
int video = 1;               /* video = 1 to print CHANDRA.TXT on screen */
			     /*  after each model, else suppress screen */
int model0 = 1;              /* initially, model0 is true */

/*                             OUTPUT VARIABLES   */
double drx[100];	      /* step size dr[m] for mth model,  */
/*           adaptively set by set_deltar(ro) with optional step divisor.  */
double massratio[100];       /* mass upper bound in solar units */
double massinner[100];       /* mass lower bound in solar units */
double radiusratio[100];     /* ratio of radii array in solar units */
			     /* minor axis ratio in rotation state */
double majorratio[100];      /* major axis ratio [to sun] in rotation */
int steps[100];              /* total steps used in the integration */
double binding_energy[100];  /* gravitational binding energy  */
double thermal_energy[100];  /* internal thermal energy */
double moment[100];          /* moment of inertia */
double redshift[100];        /* gravitational redshift from surface */
double ct[100];              /* white dwarf cooling time  */
double lumin[100];           /* white dwarf luminosity */
double teff[100];            /* white dwarf effective surface temperature */
double tc[100];              /* white dwarf central temperature */
int flyapart[100];           /* rotation flings core apart */
int pflagx[100]; /* pflagx[m] = 1 if pressure < epsilon on exit, else 0. */ 
FILE *fp2;                   /* structure file output */

void setdef(void);               /* set default variables */

int select;                      // Input routine variables
double d;                        // for crash-resistant variable input.
char value[50];			// crash-resisting string input buffer.
//       scanf("%s",value);   // This
//       d = atof(value);     // entry procedure is more immune to invalid inputs
//       select = d;          // than simple scanf().

/**************************************************
*
*   main - Enter list of starting densities and
*          condition codes for corrections, also
*          specifying whether to create stellar
*          structure files.  The output is in the
*          form of a table of starting densities,
*          radii, and masses. Optional structure
*          files give density, pressure, mass as
*          functions of the radial coordinate.
*
****************************************************/
main()
{
//                           STELLAR QUICK SKETCH ROUTINES
//
    void distancecal(void);  // cal distance r from parallax.
    void luminosity(void);   // cal luminosity L/Lsun & M from r & m.
    void stellarmass(void);  // cal stellar mass M/Msun from L/Lsun.
    void stellarradius(void); // cal R/Rsun from Tsurface and L/Lsun.
    void tsurf(void);        // cal Tsurface from spectral type.
//
//                          COLLAPSED OBJECT VARIABLES
    int v,codeval,nsec;
    int rocheck;
    int cntsuper;	// counter to init supernova vars.
    double sec, min, hours;
    double anum;
    double varb,varx;
    double angvel,tden, tpres, fxxx, f2xx, xxx, u;
    void showconditioncodes(void);
    double eos(double ro);
    double inveos(double pr);
    double cuberoot(double v);
    double f(double x);
    double f2(double z);
    double invsinh(double q);
    void restore(void);
    void polytrope(void);
    void listmainseq(void);
    int tests;
    int wdz;
    int cc;
    int rsflag;
    void changedefaults(void);
    void save(void);
    void processlist(void);
    void file_list(void);
	
    printf("\n");
    printf("CHANDRASEKHAR LIMIT SETS        Rev: 11/30/99  Greenwood Research\n");
    printf("for Windows 3.1/95/98/NT        Programmer:    James A. Green\n");
    setdef();
    printf("White dwarfs: 40 Eri B,  M = 0.45 solar masses STELLAR STRUCTURE CODE\n");
    printf("              Sirius B,  M = 1.05 solar masses        - CHANDRA -\n");
    printf("              Procyon B, M = 0.65 solar masses\n");
    printf("              10**6 < Central densities < 10**11 gr/cm**3\n");
    printf("Pulsars:      0531+21[Crab]          30.21  Hz    NEUTRON STARS\n");
    printf("              0833-45[Vela]          11.21  Hz\n");
    printf("              0525+21                0.267  Hz\n");
    printf("              1913+16[binary pulsar] 16.949 Hz\n");
    printf("              1859+54                1.526  Hz\n");
    printf("              0329+54                1.3996 Hz\n");
    printf("              1929+10                4.415  Hz\n");
    printf("              10**13 < Central densities < 10**16 gr/cm**3\n");

    printf("Main Sequence: Sun, M= 1 solar mass, use gamma = 4/3.\n");
    printf("Basic results for the models you define are written to chandra.txt.\n");
    printf("Structure variables from the center to the surface are written\n");
    printf(" to stellara.txt, should you choose to record this extra data.\n");
    printf("THESE FILES ARE REWRITTEN when chandra.exe is executed, so save files.\n");

    setdef(); /* preset variables */


/*                                                             */
/* TEST PROBES section.  To disable, let tflag = 0, tests = 0  */
/*                                                             */

     tests = 0;                 /* TEST INVERSE SINH */
     while(tests == 1){
	printf("Probe invsinh[x]. Enter x: \n");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        u = d;               // than simple scanf().

	printf("\nx = %e, invsinh[x] = %e\n", u,invsinh(u));
	printf("Enter 1 to retest else, 0: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        select = d;          // than simple scanf().
        tests = select;
	printf("\n");
      }

      tests = 0;     /* TEST log10 and exp */
      while(tests == 1){
	  printf("exp[ B*log10[x]] test.\n" );
	  printf("enter B: "); scanf("%lf",&varb); printf("\n");
	  printf("enter x: "); scanf("%lf",&varx); printf("\n");
	  varb = varb*varx;
	  varb = exp(varb);
	  printf(" exp[ B*log10[x]] = %e\n",varb);
	  printf("enter 1 to retest, 0 to advance: ");
	  scanf("%d",&tests); printf("\n");
      }

    tflag = 0;                  /* TEST CUBE ROOT */
    while(tflag == 1){
      printf("Test cube root.  Enter sci notation test value:  \n");
      scanf("%lf",&tstx);
      printf("\n\n");
      printf("Entered %e\n", tstx);
      tstx = cuberoot(tstx);
      printf("cube root = %e\n", tstx);
      printf("\nEnter 1 to retest, 0 to go on:\n");
      scanf("%d", &tflag);
      printf("\n\n");
    }

    setdef();  /* re-preset cal & dislay steps, conditions */

    tflag = 0;               /* TEST STATE INTEGRAL */
    while(tflag == 1){
       printf("\n\nTest f(x) & f2(x) in eqn of state.\n\n");
       printf("Enter floating point x : ");
       scanf("%lf",&xxx);
       fxxx = f(xxx);
       f2xx = f2(xxx);
       printf("x = %e,  f[x] = %e\n", xxx, fxxx);
       printf("        f2[x] = %e\n\n",f2xx);
       printf("Enter 1 to repeat, 0 to go on: ");
       scanf("%d",&tflag);
       printf("\n\n");
    }

    tflag = 0;               /* TEST EOS, INVERSE EOS */
    while(tflag == 1){
       printf("\n Test Eos and Inverse Eos. \n");
       printf("Enter condtion code: "); scanf("%d",&cc); printf("\n");
       m = 0;
       condition[m] = cc;
       printf("Test eos. Enter density: \n");
       scanf("%lf", &tden);
       printf("\n");
       tpres = eos(tden);
       printf("ro = %e, p[ro] = %e\n",tden,tpres);

       printf("\n\nTest inveos. Enter p: ");
       scanf("%lf",&tpres);
       printf("\n");
       tden = inveos(tpres);
       printf("ro[p] = %e, p = %e\n\n",tden,tpres);
       printf("\nEnter 1 to retest, else 0: ");
       scanf("%d",&tflag);
       printf("\n\n");
    }
//
// End of TEST PROBES Section.
//
    setdef();   /* represet variable for run */

    printf("Enter 0 for white dwarf model calculations,\n");
    printf("      1 for supernova core or neutron object model calculations,\n");
    printf("      2 for restart from interrupted execution of old model set,\n");
    printf("      3 to list typical Main Sequence stars.\n");
    printf("      4 for a STAR SKETCH of an Eddington Standard Model Star: ");

    scanf("%s",value);   // This
    d = atof(value);     // entry procedure is more immune to invalid inputs
    novas = d;      // than simple scanf().

    printf("\n\n");
    if(novas == 1){    // supernova core pre-initialize
        cntsuper = 0;
	while(cntsuper < 100){
	   dwarfz[cntsuper] = 26; // initialize supernova iron core Z=26
	   cntsuper++;
	}
    }
//
//  If stellar quick sketch is desired, sketch until abort is requested.
//
    if(novas == 3 || novas == 4){
      sketchflag = 1;
      while(sketchflag == 1){
           if(novas == 3){
              listmainseq();  /* show selection of main sequence stars. */	      
           }
           if(novas == 4){  /* polytrope quick sketch - investigate basics */
              polytrope();
	   }
      }
      printf("Exit CHANDRA");
	      exit(0);
    }

    if(novas == 2)
	 restore();

    changedefaults();

    printf("The structure file is STELLARA.TXT\n");
    showconditioncodes(); /* show code meanings */

    printf("For each line: Enter central density, conditioncode,");
    printf(" structurecal = 1 or 0\n");
    printf("             1 to continue, 0 to stop series\n");

    printf("Structure cal = 0, -> CHANDRA.TXT,");
    printf(" = 1 -> STELLARA.TXT,CHANDRA.TXT\n");

    gocode = 1;
    printf("Enter list of central densities, limit 100. \n");
    printf("Typical WD to Neutron star densities: 1.0e7 < ro < 1.0e16 gr/cm**3\n");
    printf("Solar central density = 7.635e1 gr/cm**3. Try dwarf 5.0e8 gr/cm**3\n");
    j = 0;                      /* INPUT DATA ENTRY LOOP */
    while(gocode == 1){
       rocheck = 1;
       while(rocheck == 1){
	   printf("Enter the central density for model=%d in scientific notation:",j);

	   scanf("%s",value);   // This
           den = atof(value);     // entry procedure is more immune to invalid inputs

	   printf("entered %e.\n", den);
	   if( den < 1.1e20 ){
	      density[j] = den;
	      rocheck = 0;
	   }
	   if( den >= 1.1e20 ){
	      printf("\nError: enter a density < 1.1e20 \n");
	      rocheck = 1;
	   }
       }

	if(rotate == 1){       /* enter rotation characteristics */
	   printf("Enter floating-point rotation in revolutions/sec [try 0.0]: ");
	   scanf("%s",value);   // This
           angvel = atof(value);     // entry procedure is more immune to invalid inputs

	   rotation[j] = angvel*6.28318;
	   printf("entered %e rev/sec\n", angvel);
	   printf("        %e rad/sec\n",angvel*6.28318);
	}

	if(novas == 0){       /* enter white dwarf composition */
	   printf("Chemistry Select: Z[C,Mg,Si,S,Fe] = 6,12,14,16,26\n");
	   printf("Enter integer typical nuclear Z for this white dwarf model:  ");
	   scanf("%s",value);   // This
           d = atof(value);     // entry procedure is more immune to invalid inputs
           wdz = d;      // than simple scanf().

	   dwarfz[j] = wdz;
	   printf("       entered Z = %d for Wigner-Seitz corr.\n",wdz);
	   printf("Chemistry, atomic number A[C,Mg,Si,S,Fe] = 12.0,24.0,26.0,30.0,56.0\n");
	   printf("Enter floating-point nuclear A for white dwarf model:  ");
	   scanf("%s",value);   // This
           anum = atof(value);     // entry procedure is more immune to invalid inputs

	   dwarfa[j] = anum;
	   printf("       entered A = %lf \n",anum);
	}

	printf("ENTER condition code, [0 is the original used by Chandrasekhar]\n");
	printf("0=unscreened with Newtonian gravity, ignores elec. pressure screening,\n");
	printf("1=unscreened with General Relativistic gravity,\n");
	printf("2=pressure electrostatically screened and using Newtonian gravity,\n");
	printf("3=pressure electrostatically screened using General Relativity,\n");
	printf("4 = Newtonian, with neutronization,   5 = GR, with neutronization,\n");
        printf("6 = Newtonian, with electrostatic screening and neutronization,\n");
        printf("7 = GR, with electrostatic screening and neutronization,\n");
	printf("8 = GR, with purely neutrons,\n");
        printf("9 = Harrison-Wheeler, with Newtonian gravitation.\n");
        printf("10 = Harrison-Wheeler EOS, general relativistic gravitation.\n");
        printf("11 = Harrison-Wheeler, GR gravity, with elect. pressure screening.\n");
        printf("12 = Baym-Bethe Equation Of State,\n");
        printf("13 = Bowers & Deeming non-relativistic degenerate neutron gas,\n");
        printf("14 = Bowers & Deeming non-rel. degen. neutron gas with quark limit,\n");
        printf("15 = Neutron object as a non-screened D-Quark gas\n");
        printf("16 = Neutron object as U-quark-screened D-Quark gas\n");
	printf("17 = Relativistic Neutron gas with Chandra-like Eqn. Of State,\n");	
	printf("Defined condition codes include 0-17 [try 0]: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        codeval = d;      // than simple scanf().

	condition[j] = codeval;
	printf("\n");

	if(condition[j] == 15 || condition[j] == 16){
		printf("Enter alpha for D-Quark mass = alpha x neutron_mass: ");
		scanf("%s",value);   // This
       		alpha = atof(value);     // entry procedure is more immune to invalid inputs

		alphax[j] = alpha; printf("\n");
        }

	printf("Enter 1 to make a big structure file, else 0 for chandra.txt only: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        codeval = d;      // than simple scanf().

	calstruc[j] = codeval;
	printf("\n");
//
/* TIME PER LOOP on old home 80386 was 12 sec/loop  */
/*               on new Pentium at 233 Mhz with MMX was 0.109945 sec/loop */
//
	sec = (j+1)*12; /* 12 sec per loop, j+1 models of looplim loops */
	sec = sec*looplim;
	min = sec/60.0;
	hours = min/60.0;
        printf("CHANDRA can process a maximum of 100 models per run.\n");
	printf("Number of Models = %d, Approximate worst 80386 runtime = %f hours.\n",j+1,hours);

        sec = j + 1;
	sec = sec*0.109945; /* 0.11 sec per loop, j+1 models of looplim loops */
	sec = sec*looplim;
	min = sec/60.0;
	printf("Use modern laptop. Approximate 233Mhz Pentium runtime = %f min.\n\n",min);
	printf("A 233Mhz Pentium machine runs CHANDRA in a blur with a barely\n");
	printf("discernable loop counter for < 4 min. x number_models_in_list.\n\n");

	printf("Enter 1 to continue data entry for next model star, else 0: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        gocode = d;      // than simple scanf().

	printf("\n\n");
	++j;
    }

    jsave = j;
    save();               /* allow immediate restart recovery */

    m = 0;                /* initialize list pointer */
    processlist();        /* process starting densities list */

    file_list();         /* save results in chandra.txt and*/
			 /* echo the results to the screen */

    printf("Series End\n");
    exit(0);
}
/**************************************************
*
*  file_list  -  Write list of results to "chandra.txt".
*                and echo them to the screen.
*
***************************************************/
void file_list(void)
{
     int fixed;      /* fix flag for Bowers&Deeming luminosity model */
     int cc;         /* condition code */
     int cs;         /* integer calstruc dummy */
     double radkm;   /* radius in kilometers */
     double rraya;   /* radius in solar radii [major axis] */
     double rrayb;   /* minor polar axis in rotational case */
     double mraya;   /* mass in solar mass units */
     double mrayb;   /* mass lower bound in solar units */
     double be;      /* gravitational binding energy */
     double therm;   /* total internal thermal energy */
     double momenti; /* moment of inertia */
     double da;      /* dwarfa */
     int zz;          /* dwarfz, Z */
     double ellipticity;  /* e of ellipse */
     FILE *fp;
     int status;
     int m;
     int msave;
     double nxdrcent, nxdrkm; /* dr x steps in centimeters, in kilometers */

     if(video == 1)
     printf("\NBACKUP TO DISK OF PRINCIPAL RESULTS: Opening chandra.txt\n");
     fp = fopen("chandra.txt","w");

     fprintf(fp,"\n\nChandrasekhar Limit Set Results\n\n");
	 if(video == 1)
	 printf("\n\nChandrasekhar Limit Set Results\n\n");
     if(novas == 0){
	 fprintf(fp,"White Dwarf &/or Supernova Core Set Condition Code Key\n\n");
	     if(video == 1)
	     printf("White Dwarf &/or Supernova Core Set Condition Code Key\n\n");
      }
      if(novas == 1){
	 fprintf(fp,"Supernova Cores, Z = 26\n\n");
	     if(video == 1)
	     printf("Supernova Cores, Z = 26\n\n");
      }      

     fprintf(fp,"0 = newtonian gravitation,      1 = general relativity\n");
     fprintf(fp,"2 = newtonian, es-screened,     3 = gen rel, es-screened\n");
     fprintf(fp,"4 = newton, w neutronization,   5 = gr, w neutronization\n");
     fprintf(fp,"6 = newton, w screening and neutronization\n");
     fprintf(fp,"7 = gr, w screening and neutronization\n");
     fprintf(fp,"8 = gr, w purely neutrons\n");
     fprintf(fp,"9 = Harrison-Wheeler, newtonian\n");
     fprintf(fp,"10 = Harrison-Wheeler, gen. rel.\n");
     fprintf(fp,"11 = Harrison-Wheeler, gen.rel., screening.\n");
     fprintf(fp,"12 = Baym-Bethe EOS\n");
     fprintf(fp,"13 = Bowers&Deeming non-rel. degen neutron gas\n");
     fprintf(fp,"14 = Bowers&Deeming non-rel. degen n. gas w quark lim\n");
     fprintf(fp,"15 = Neutron object as non-screened D-Quark gas\n");
     fprintf(fp,"16 = Neutron object as U-quark-screened D-Quark gas\n");
     fprintf(fp,"17 = Relativistic Neutron gas with Chandra-like EOS\n");
     fprintf(fp,"Model      Number of Models = %d\n",jsave);
	 if(video == 1)
	 printf("Model      Number of Models = %d\n",jsave);
     for(m = 0; m < j; m++){
	   cc = condition[m];
	   fprintf(fp,"\nmodel=%d      condition code = %d\n",m,cc);
	       if(video == 1)
	       printf("\nmodel=%d      condition code = %d\n",m,cc);
	   fprintf(fp,"step differential dr = %e cm\n",drx[m]);
	   nxdrcent = drx[m]*steps[m];
           nxdrkm = (drx[m]*steps[m])/1000.0;
           fprintf(fp," dr x steps[m] = %e cm, %e km\n", nxdrcent, nxdrkm);
	   zz = dwarfz[m];
	   fprintf(fp,"central density = %e gr/cm3, ", density[m] );

	   fprintf(fp,"Z[start] = %d,", zz);
	       if(video == 1){
	       printf("central density = %e gr/cm3, ", density[m] );  
	       printf("Z[start] = %d,", zz);		  
	       }           
	   fprintf(fp,"  A = %f\n",dwarfa[m]);
     	   if(video == 1)
	       printf("  A = %f\n",dwarfa[m]);
	   rraya = radiusratio[m];
	   fprintf(fp,"rp =  %e solar radii", rraya);
	       if(video == 1)
	       printf("rp =  %e solar radii", rraya);
	   fprintf(fp,"  =  %e km\n",rraya*rsolkm);
	       if(video == 1)
	       printf("  =  %e km\n",rraya*rsolkm);
	   if(condition[m] == 15 || condition[m] == 16){
		fprintf(fp,"D-quark alpha = %f\n",alphax[m]);
		printf("D-quark alpha = %f\n",alphax[m]);
	   }
	   if(pflagx[m] == 1){
		fprintf(fp,"Pressure < epsilon on exit from integration.\n");
	   }
	   if(pflagx[m] == 0){
		fprintf(fp,"Pressure not < epsilon on exit from integration.\n");
           }
	   mraya = massratio[m];
	   mrayb = massinner[m];
	   fprintf(fp,"M [upper limit] = %e solar masses  \n", mraya);
	       if(video == 1)
	       printf("M[upper limit]= %e M_sun, ", mraya);
	   fprintf(fp,"M [lower limit] = %e solar masses\n",mrayb);
	       if(video == 1)
	       printf("M[lower limit]= %e M_sun.\n",mrayb);
	   cs = calstruc[m];
	   fprintf(fp,"structure file stored =  %d\n", cs);
	       if(video == 1)
	       printf("structure file stored =  %d  ", cs);
	   fprintf(fp,"steps = %d  \n",steps[m]);
	       if(video == 1)
	       printf("steps = %d  \n",steps[m]);
	   be = binding_energy[m];
	      fprintf(fp,"binding energy = %e ergs\n",be);
		  if(video == 1)
		  printf("binding energy = %e ergs\n",be);

	   therm = thermal_energy[m];
	   fprintf(fp,"internal thermal energy = %e ergs\n",therm);
	       if(video == 1)
	       printf("internal thermal energy = %e ergs\n",therm);
	   momenti = moment[m];
	   fprintf(fp,"moment of inertia [cgs] = %e \n",momenti);
	       if(video == 1)
	       printf("moment of inertia [cgs] = %e \n",momenti);
	   if(rotate == 1){
	       fprintf(fp,"angular rev/sec = %f \n", rotation[m]/6.28318);
		   if(video == 1)
		   printf("angular rev/sec = %f  ", rotation[m]/6.28318);
	       fprintf(fp,"ang.velocity = %e  rad/sec\n",rotation[m]);
		   if(video == 1)
		   printf("ang.velocity = %e  rad/sec\n",rotation[m]);
	       fprintf(fp,"polar R in km = %e\n",rraya*rsolkm);
		   if(video == 1)
		   printf("polar R in km = %e\n",rraya*rsolkm);
		   rrayb = majorratio[m];
	       fprintf(fp,"equitorial R in km = %e  ",rrayb*rsolkm);
		   if(video == 1)
		   printf("equitorial R in km = %e  ",rrayb*rsolkm);
// flag suspected rotating star problem statement - fix domain error on sqrt.
               if(rrayb*rrayb - rraya*rraya >= 0.0 )
		   ellipticity = sqrt( (rrayb*rrayb - rraya*rraya) )/rrayb;
	       else {
		   ellipticity = 0.0;
		   printf(" Fast Rotation Flyapart Exception.\n");
	       }
	       fprintf(fp,"ellipticity e = %f \n",ellipticity);
		   if(video == 1)
		   printf("ellipticity e = %f \n",ellipticity);

	       fixed == 0;  /* fix flag for special quantities */
	       if(novas == 0 && fixed == 1){
		 fprintf(fp,"dwarf central temperature < %e  K\n",tc[m]);
		   if(video == 1)
		   printf("dwarf central temperature < %e  K\n",tc[m]);
		 fprintf(fp,"white dwarf luminosity = %e solar L\n",lumin[m]);
		   if(video == 1)
		   printf("white dwarf luminosity = %e solar L\n",lumin[m]);
		 fprintf(fp,"white dwarf surface temperature = %e K\n",teff[m]);
		   if(video == 1)
		   printf("white dwarf surface temperature = %e K\n",teff[m]);
		 fprintf(fp,"white dwarf cooling time = %e years\n",ct[m]);
		     if(video == 1)
		     printf("white dwarf cooling time = %e years\n",ct[m]);
	       }
	       fprintf(fp,"flyapart = %d\n",flyapart[m]);
		   if(video == 1)
		   printf("flyapart = %d ",flyapart[m]);
	       fprintf(fp,"redshift = %e\n\n",redshift[m]);
		   if(video == 1)
		   printf("redshift = %e\n",redshift[m]);
	   }
     }
     fprintf(fp,"\nLooplim = %d\n",looplim);
	 if(video == 1)
	 printf("Looplim = %d\n",looplim);
     fprintf(fp,"Step divisor = %d, rotate = %d\n",step_size,rotate);
	 if(video == 1)
	 printf("Step divisor = %d, rotate = %d\n",step_size,rotate);
     fprintf(fp,"Quark EOS above 1.44 x 10**16 = %d\n",quarks);
	 if(video == 1)
	 printf("Quark EOS above 1.44 x 10**16 = %d\n",quarks);
     if(greens == 0 && rotate == 1 && video == 1){
	 printf("Spherically symmetric grav field + centrifugal potential\n");
	 printf(" approximation. Fast calculation. Good until appreciable\n");
	 printf(" deviations from spherical symmetry occur.\n");
     }
     if(greens == 1 && rotate == 1 && video == 1){
	 printf("Green's functions used calculating gravitational field\n");
	 printf(" + centrifugal potential. Good for appreciable deviation\n");
	 printf(" from spherical symmetry. Time-consuming calculation.\n\n");
     }

     fprintf(fp,"\n\nINITIAL SETUP. %d models in the set.\n\n",jsave);
     for(m = 0; m < jsave; m++){
	fprintf(fp,"model %d   ro0 = %e, ", m, density[m]);
	fprintf(fp,"condition_code = %d ", condition[m]);
	da = dwarfa[m];
	fprintf(fp," Z = %d, A = %f \n", dwarfz[m], da);
	fprintf(fp,"    calstruc = %d, ", calstruc[m]);
	fprintf(fp,"rev/sec = %f\n", rotation[m]/6.28318);
     }
     fprintf(fp,"\n\n");
     status = fclose(fp);
     if(video == 1)
     printf("THE SUMMARY RESULTS WERE WRITTEN INTO chandra.txt.\n");
     printf("  File chandra.txt was written to the current chandra.exe directory.\n");
     printf("  All detailed center-to-surface stucture is written into stellara.txt.\n");
}
/**************************************************
*
*  setdef - Preset calculation and display steps,
*           mass and radius ratios.
*
***************************************************/
void setdef(void)
{
     int kk;

     for(kk = 0; kk < 100; kk++){ /* preset ratios */
	 density[kk] = 0.0;
	 rotation[kk] = 0.0;
         alphax[kk] = 0.3333;
	 massratio[kk] = 0.0;
	 massinner[kk] = 0.0;
	 radiusratio[kk] = 0.0;
	 condition[kk] = 0;
	 calstruc[kk] = 0;
	 steps[kk] = 0;
	 binding_energy[kk] = 0.0;
	 thermal_energy[kk] = 0.0;
	 moment[kk] = 0.0;
	 redshift[kk] = 0.0;
	 majorratio[kk] = 0.0;
	 flyapart[kk] = 0;
	 dwarfz[kk] = 26;
	 dwarfa[kk] = 56.0;
	 ct[kk] = 0.0;
	 lumin[kk] = 0.0;
	 tc[kk] = 0.0;
	 teff[kk] = 0.0;
         pflagx[kk] = 0;	/* assume press < epsilon not taken */
     }
}
/***********************************************
*
*  show_condition_codes() - Display the condition
*                           code meanings.
*
************************************************/
void showconditioncodes(void)
{
     
     printf("\n 0 - Newtonian gravity, unscreened electron gas. ");
     printf("   CONDITION CODES\n");
     printf(" 1 - General relativistic gravity only.\n");
     printf(" 2 - Newtonian, es-screened,         3 - Gen Rel, es-screened\n");
     printf(" 4 - Newtonian,neutronization,       5 - Gen Rel, neutronization,\n");
     printf(" 6 - Newtonian,neutroniz.&screening. 7 - Gen Rel, neut.& screening\n");
     printf(" 7 - General relativity, neutroniz. & elec. screening.\n");
     printf(" 8 - General relativity, pure Fermi neutron gas.\n");
     printf(" 9 - Harrison-Wheeler, Newtonian gravity\n");
     printf("10 - Harrison-Wheeler EOS, general relativity\n");
     printf("11 - Harrison-Wheeler EOS, general rel., screening.\n");
     printf("12 - Baym-Bethe-Pethick EOS, lin.approx.,gen. relativity.\n");
     printf("13 - Bowers&Deeming nonrel. degen. neutron gas, GR.\n");
     printf("14 - Bowers&Deeming nonrel. degen. n. gas, GR, & quark limit\n");
     printf("15 - Neutron object as non-charge-screened D-Quark gas.\n");
     printf("16 - Neutron object as charge-screened D-quark gas.\n");
     printf("17 - Relativistic Neutron gas with Chandra-like eos.\n");    
}
/***********************************************
*
*  changedefaults - Enter new cal_step or
*                    display_step values.
*
*************************************************/
void changedefaults(void)
{
     int swit;
     int rot;
     double ss,dr;

     printf("ADAPTIVE STEP WIDTH CONDITIONS FOR INTEGRATION\n");
     printf("Default pre-selected radial step divisor   = %d.\n",step_size);
     ss = step_size;
     dr = 5.0/ss;   /* see set_deltar for adaptive size settings as below; */
     printf("Radial step adaptive.\n");
     printf("  Typical neutron star radius = 45 km\n");
     printf("  Typical white dwarf radius =  3331 km\n");
     
     if( novas == 0 || novas == 1 ){
       printf("   For          ro < 10**9,  dr = 10.0 km/(step divisor).\n");
       printf("   For 10**9  < ro < 10**10, dr =  5.0 km/(step divisor).\n");
       printf("   For 10**10 < ro < 10**12, dr = 5.0 km/(step divisor).\n");
       printf("   For 10**12 < ro < 10**13, dr = 2.0 km/(step divisor).\n");
       printf("   For 10**13 < ro < 10**14, dr = 2.0 km/(step divisor).\n");
       printf("   For 10**14 < ro < 10**15, dr = 0.5 km/(step divisor).\n");
       printf("   For 10**15 < ro < 10**17, dr = 0.1 km/(step divisor).\n");
       printf("   For 10**17 < ro < 10**19, dr = 0.1 km/(step divisor).\n");
       printf("   For 10**19 < ro,          dr = 0.05 km/(step divisor).\n");
     }
     printf("Limiting number of radial steps per model = %d.\n",looplim);
     printf("Enter 1 to modify, else 0 for the recommended default: ");
     scanf("%s",value);   // This
     d = atof(value);     // entry procedure is more immune to invalid inputs
     swit = d;      // than simple scanf().

     if(swit == 1){
	printf("Default loop limit is %d. Enter new limit: ",looplim);
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        looplim = d;      // than simple scanf().

	printf("\nEnter radial step integer divisor, 1,2,3,4,5,... ");
	printf("\nSuggested value is 5: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        step_size = d;      // than simple scanf().

	printf("\n\n");
     }
     printf("Enter 1 for limiting with quark-model EOS, else 0 regulation default: ");
     scanf("%s",value);   // This
     d = atof(value);     // entry procedure is more immune to invalid inputs
     quarks = d;      // than simple scanf().

     printf("\n\n");
     printf("Rotating models used. ");
     rotate = 1;
     greens = 0;     /* use approximate rotational model */
/*   if(rotate == 1){
	 printf("Enter 0 for spherically symmetric field modified by\n");
	 printf("a centrifugal potential.  [A FAST cal method.]    OR \n");
	 printf("Enter 1 for a Green's function over the ellipsoidal\n");
	 printf("shells plus the centrifugal potential.[ SLOW cal.]:");
	 scanf("%s",value);   // This
         d = atof(value);     // entry procedure is more immune to invalid inputs
         greens = d;      // than simple scanf().

	 printf("\n\n");
	 greens = 0;
     }
		      */
}
/**********************************************
*
*  processlist() - Process the list of starting
*                   densities and condtion specifiers.
*   in: j = points to last entry + 1.
*  out: m = universally visible list pointer during
*           subroutine invocations.
*
************************************************/
void processlist(void)
{
    double denx;
    double a,b;
    void process(void);

    printf("\n\n STARCORE DATA STREAM \n\n");
    if(quarks == 1)
	printf("Quark model = %d\n",quarks);
    if(rotate == 1)
	printf("Rotations are on.\n\n");
    while( m < j ){
	process();
	printf("\n\nSTARCORE MODEL = %d of %d models\n",m,j);
	denx = density[m];
	printf("Starting density = %e  grams per cubic cm\n",denx);
	printf("Rotational velocity = %e rev/sec\n", rotation[m]/6.28318);
	printf("Steps = %d\n",steps[m]);
	printf("Structure file stored = %d\n",calstruc[m]);
	printf("Radius in solar units = %e\n",radiusratio[m]);
	printf("MASS in solar units   = %e\n",massratio[m]);
	printf("MASS lower limit      = %e\n",massinner[m]);
	printf("Cond.code = %d\n",condition[m]);
	printf("polar axis = %e km, ", rsolar*radiusratio[m]/100000.0);
	printf("equitorial axis = %e km\n", rsolar*majorratio[m]/100000.0);
	a = majorratio[m]; b = radiusratio[m];
        if( a*a - b*b >= 0.0)
	    printf("Ellipticity = %e\n", sqrt( a*a - b*b )/(b*b) );
	else{
	    printf("Fast rotation domain error. Ellipticity undefined.\n");
	}
	printf("Binding energy = %e\n",binding_energy[m]);
	printf("Thermal energy = %e\n",thermal_energy[m]);
	printf("Moment of inertia = %e\n",moment[m]);
	printf("Flyapart = %d\n",flyapart[m]);
	printf("Redshift = %e  END MODEL %d of %d.\n\n",redshift[m],m,j);
	m++;
    }
}
/**************************************************
*
*  process - Process the list of variables.
*
* in: density[m] = densities,
*     condition[m] = condition codes
*                    0 = newtonian only
*                    1 = general relativity
*                    2 = newtonian, eos corrections
*                    3 = gr, eos corrections
*                    4 = newtonian, neutronization
*                    5 = gr, neutronization
*                    6 = newtonian, eos, neut.
*                    7 = gr, eos, neut.
*     calstruc[m] = go code for structure cal.
*                   0 = no struc cal.
*                   1 = structural cal.
*
****************************************************/
void process(void)
{
    double rray, mray;
    double rkm;
    void starcal(void);
    void file_list(void);
    void save(void);

    starcal();  /* perform model calculation */
    model0 = 0;
    jsave = j;
    j = m + 1;
    video = 0;
    printf("BUILDING CHANDRA.TXT & RESTART SAVE FILES. DO NOT INTERRUPT.\n");
    file_list(); /* save CHANDRA.TXT through the last processed model. */
    save();
    printf("CHANDRA.TXT REPORT FILE M SAVED, also RESTART.TXT.\n");
    video = 1;
    j = jsave;

}
/***********************************************
*
*  starcal - Process density[m] starting densities
*            with condition[m] and use calstruc[m]
*            = 1 or 0 to compute or not to compute
*            a stellar structure file.
*
**************************************************/
void starcal(void)
{
    void newtonian(void);
    void genrel(void);
    radiusratio[m] = 1.0;   /* defaults */
    massratio[m] = 1.0;     /* mass default */

    if(condition[m] == 0)
       newtonian();        /* simple Newtonian model */
    if(condition[m] == 1)
       genrel();          /* General Relativity w. Oppenheimer-Volkoff eqn */
    if(condition[m] == 2)
       newtonian();        /* Newtonian w Electrostatic screening */
    if(condition[m] == 3)
       genrel();          /* GR w Electrostatic screening corrections */
    if(condition[m] == 4)
       newtonian();            /* Newt. with neutronization */
    if(condition[m] == 5)
       genrel();            /* GR with neutronization */
    if(condition[m] == 6)
       newtonian();           /* Newt. with neut. and elec. screening */
    if(condition[m] == 7)
       genrel();            /* GR with neutr. & elec. screening */
    if(condition[m] == 8)
       genrel();           /* Pure Fermi neutron gas w/o protons - Oppie */
    if(condition[m] == 9)
       newtonian();        /* Harrison-Wheeler in newtonian */
    if(condition[m] == 10)
       genrel();           /* Harrison-Wheeler in gen rel. */
    if(condition[m] == 11)
       genrel();           /* Harrison-Wheeler with screen in gen. rel */
    if(condition[m] == 12)
       genrel();           /* Baym-Bethe EOS in gen rel */
    if(condition[m] == 13)
       genrel();           /* Bowers&Deeming EOS in general relativity */
    if(condition[m] == 14)
       genrel();           /* Bowers&Deeming EOS in GR w. Zeldovich quark lim */
    if(condition[m] == 15)
	newtonian();	  /* D-Quark sea neutron object, no screening */
    if(condition[m] == 16)
	newtonian();	   /* D-Quark sea neutron object with screening */
    if(condition[m] == 17)
	newtonian();	   /* Relativitistic neutron sea */
}
/*****************************************
*
*  newtonian - Integrate hydrostatic equilibrium
*              equation from the center of the
*              star out, using an equation of state
*              that automatically introduces
*              Wigner-Seitz electrostatic shielding
*              if requested based on condition[m].
*              A structure file stellara.txt will
*              be produced if calstruc[m] = 1.
*              If rotate = 1, integrate along major axis,
*              cal minor axis, sum ellipsoidal shells.
*              When figuring total white dwarf thermal energy,
*              present software uses rigorous energy density.
*              Rotating star calculations are only
*              accurate until the major and minor
*              axes differ appreciably.
*              Integration of mass proceeds by lower and upper
*              sums for precise limiting mass investigations.
*
*  fault note: Rotations rapid enough to distort the object with
*              ellipticity > 0.998 can lead to a sqrt domain error.
*
*******************************************/
void newtonian(void)
{
     int fixed;            /* fix flag for Bowers&Deeming luminosity model */
     int status;           /* status after closing struc file */
     double anuc;          /* nucleons in nucleus */
     double znuc;          /* nuclear charge */
     double lum;           /* luminosity */
     double tc1,t1,t2,t3;  /* temperature variables for luminosity */
     double dr,dre;        /* polar & equitorial radius differentials */
     double mr, delm;      /* mass[r] for upper sum integral, and dm[r] */
     double mr_in;         /* mass[r] for lower sum integral */
     double ro, ro0;       /* starting density */
     double omega;         /* angular velocity of rotation */
     int jm;               /* index for shell steps from center */
     int  trace;           /* trace flag */
     double rn, rne;       /* minor and major axis coordinates */
     double rn_1, rne_1;   /* minor and major axis, previous step */
     double dvol, dvol_1;  /* present volume, previous step volume */
     double g = 6.67e-8;   /* Newtonian gravitation constant */
     double c = 2.9979e10; /* light speed */
     double p0, p, delp;   /* initial pressure, pressure[r], dp */
     double epsilon;       /* cut-off pressure at stellar surface */
     double epsilonro;     /* cut-off density at stellar surface */
     double momenti;       /* moment of inertia at r */
     double scale;         /* scaler for equitorial radius */
     double binding;       /* binding energy */
     double thermal;       /* thermal energy */
     double cuberoot(double ro);    /* cube root */
     double seventhroot(double ro); /* seventh root */
     double eos(double v);         /* equation of state */
     double inveos(double pr);      /* inverse equation of state */
     double thermen(double ro, double p);     /* energy per unit volume for electron gas */
     double set_deltar(double ro);  /* adaptive step-size routine */
     int pflag;		/* pflag = 1 if exit due to pressure < epsilon */

     pflag = 0;			/* pflag = 1 when pressure < epsilon on exit */
     trace = 1;                /* trace variable cal on screen */
     p0 = eos(density[m]);     /* cal central pressure */
     p = p0;                   /* init p[r] */
     epsilon = 1.0;            /* init pressure cutoff at surface */
     epsilonro = 0.001;        /* init density cutoff at surface */
     ro = density[m];          /* init ro[r] */
     ro0 = density[m];         /* init central density */
     ro00 = ro0;		/* init global var for current central density */
     alpha = alphax[m];		/* init Dquark_mass = alpha x neutron mass */
     anuc = dwarfa[m];         /* nucleon number */
     znuc = dwarfz[m];         /* nuclear charge */
     tc[m] = (hbar*c/(kb*cuberoot(anuc*mproton)))*cuberoot(ro0);  // WD central temp.
     omega = rotation[m];      /* init angular velocity */
     deltar = set_deltar(ro);  /* set step size deltar, deltar2 adaptively */
     rn =  deltar;             /* initialize radius */
     rne = 0.0 - 1.0;          /* set-up for rot. initialization later */
     dr = deltar;              /* init polar radius differential */

     dvol = (4.0/3.0)*3.14159*(rn*rn*rn);
     dvol_1 = dvol;
     mr = dvol*ro0;                                 /* set m[0], outer sum */
     thermal = thermen(ro,p)*dvol;                  /* set thermal[0] */
     binding = (3.0/5.0)*(g*mr*mr/rn);              /* set binding[0] */
     momenti = (2.0/5.0)*mr*rn*rn;                  /* set moment[0] */

     if(calstruc[m] == 1){
	if(openflag == 0){                      /* open structure file */
	  fp2 = fopen("stellara.txt", "w");
	  openflag = 1;
	}
	fprintf(fp2,"\n\nNewtonian structure.  Central density = %e\n",ro);
	fprintf(fp2,"Cond.code = %d  ", condition[m]);
	fprintf(fp2,"Omega = %e rad/sec, rotate =%d\n", omega,rotate);
	fprintf(fp2,"Stellar Model Number = %d\n\n",m);
	fprintf(fp2,"density      radius/Rsolar     mass/Msolar  ");
	fprintf(fp2,"pressure \n\n");
     }
					    /* loop limit default = 2001 */
     mr_in = 0.0; /* init inner sum integral */
     jm = 0;
     while( p > epsilon && jm < looplim){   /* cut off in tenuous envelope */
	delp = (g*mr/(rn*rn))*ro*dr;
	if(rotate == 1){
	   scale = ((g*mr/(rn*rn)) - (omega*omega*rn));
	   if(scale <= 0.0){
	      flyapart[m] = 1;
	      delp = 0.0;
	      p = 0.0;           /* force p < epsilon = 1 dyne/cm3 */
	   }
	   if(scale > 0.0 && ro > epsilonro)
	       dre = delp/(scale*ro);    /* equitorial axis differential */
	   if(rne < 0.0){                /* special initizations */
	       rne = dre;                /* initialize equitorial radius. */
	       dvol = (4.0/3.0)*(3.14159)*rn*rne*rne;
	       dvol_1 = dvol;   /* init core volume */
	       mr = dvol*ro0;
	   }
	}
	if(trace == 1){
	    printf("model%d of %d loop%d, ro0 = %e gr/cm**3\n",m,j,jm,ro0);
	    printf("density = %e, p = %e, dp = %e\n",ro,p,delp);
	}
	p = p - delp;
	if(p > epsilon){      /*  cut off in thin envelope */
	   ro = inveos(p);
           pflag = 0;
	}
	else{
	   ro = 0.0;
           pflag = 1;
	   p = 0.0;
	}
	rn_1 = rn;
	rne_1 = rne;
	rn = rn + dr;
	rne = rne + dre;
	if(rotate == 0){
	  dvol = 4.0*3.14159*(rn_1*rn_1)*dr;
	  delm = dvol*ro; /* upper sum delm */
	  binding = binding + (g*mr/rn_1)*(delm);
	  thermal = thermal + (thermen(ro,p))*(dvol);
	  momenti = momenti + (2.0/3.0)*delm*(rn_1 + dr/2.0)*(rn_1 + dr/2.0);
	  mr = mr + delm;             /* upper sum for mass integral */
	  mr_in = mr_in + dvol_1*ro;  /* lower sum for mass integral */
	  if(trace == 1){
		 printf("mr[upper bound] = %e solar masses\n",mr/msolar);
		 printf("mr[lower bound] = %e solar masses\n",mr_in/msolar);
		 printf("r = %e km\n",rn);
		 printf("thermal energy[r] = %e ergs\n",thermal);
		 printf("binding energy[r] = %e ergs\n\n",binding);
	   }
	}
	if(rotate == 1){   /* use volume difference between two ellipsoids */
	  dvol = (4.0/3.0)*(3.14159)*(rne_1 + dre)*(rne_1 + dre)*(rn_1 + dr);
	  dvol = dvol - (4.0/3.0)*3.14159*(rne_1*rne_1*rn_1);
	  delm = dvol*ro;
	  thermal = thermal + (dvol)*(thermen(ro,p));
	  binding = binding + (g*mr/(rn_1+rne_1)/2.0)*(delm);
	  momenti = momenti + (2.0/3.0)*(delm*((rn + rne)/2.0))*((rn+rne)/2.0);
	  mr = mr + delm;            /* outer sum mass integral */
	  mr_in = mr_in + dvol_1*ro; /* inner sum mass integral */
	  if(trace == 1){
	     printf("mr[upper bound] = %e solar masses\n",mr/msolar);
	     printf("mr[lower bound] = %e solar masses\n",mr_in/msolar);
	     printf("major axis r = %e cm, rotation axis = %e cm\n",rne,rn);
	     printf("moment_of_inertia[r] = %e\n", momenti);
	     printf("thermal_energy[r] = %e ergs\n",thermal);
	     printf("binding energy[r] = %e ergs\n\n",binding);
	  }
	}
	dvol_1 = dvol;
	jm = jm + 1;
	if(calstruc[m] == 1){
	   fprintf(fp2," %e   %e    %e    %e\n", ro,rn/rsolar,mr/msolar,p);
	}
     }
     printf("\nPrepare summary calculations.\n");  /* cal summary */
     drx[m] = dr;		/* save step diff. dr in cm */
     radiusratio[m] = rn/rsolar;
     majorratio[m] = rne/rsolar;
     massratio[m] = mr/msolar;       /*  mass upper bound */
     massinner[m] = mr_in/msolar;    /*  mass lower bound */
     redshift[m] = (g*mr/rn)/(c*c);  /*  dv/v = -dlamda/lamda  redshift */
     binding_energy[m] = binding;
     thermal_energy[m] = thermal;
     moment[m] = momenti;
     steps[m] = jm;
     pflagx[m] = pflag;  /* pflag = 1 if pressure < epsilon on exit,else 0 */
     lumin[m] = 0.0;
     ct[m] = 0.0;

     fixed = 0;    /* Bowers & Deeming luminosity model doubtful */
     if(looplim > 100 && fixed == 1){ /* luminosity models */
       printf("Compute white dwarf luminosity.\n");
       tc1 = tc[m];        /* white dwarf luminosity */
       if(tc1 >= 0.0)
	  t1 = sqrt(tc1);
       else t1 = 0.0;
       t1 = t1*t1*t1*t1*t1*t1*t1;
       lumin[m] = (5.7e25)*(10.0/znuc)*(anuc/4.0)*(mr/msolar)*t1; /* white dw */
       lum = lumin[m];
       printf("white dwarf luminosity = %e ergs\n",lum);
       if( lum/(4.0*3.14159*sigma*rn*rn) >= 0.0 )
	   teff[m] = sqrt( sqrt( lum/(4.0*3.14159*sigma*rn*rn) ) ); /* surface T */
       else teff[m] = 0.0;
       lumin[m] = lum/lsolar;

       printf("Cal white dwarf cooling time.\n");
       t1 = seventhroot( (mr/msolar)*(lsolar/lum) ); /* cal cooling time */
       t1 = t1*t1*t1*t1*t1;
       t2 = seventhroot( ((4.0/anuc)*(0.1)*znuc) );
       t2 = t2*t2/anuc;
       ct[m] = (8.9e7)*t2*t1;  /* cooling time in years, white dwarf */
     }

     if(openflag == 1 && m == j - 1){
	 fprintf(fp2,"\n\n");
	 status = fclose(fp2);
	 printf("\nClose STELLARA.TXT\n");
     }
}
/*****************************************************************
*
*  genrel - Integrate the GR-valid Tolman-Oppenheimer-
*            Volkoff equation of hydrostatic equilibrium
*            from the core out, until P = 0.
*            Approximate treatment of rotations until major & minor
*            axes differ appreciably.
*
******************************************************************/
void genrel(void)
{
    int fixed;                   /* fix flag for Bowers&Deeming luminosity */
    int status;                  /* status after closing struc file */
    double znuc;                 /* nuclear charge */
    double anuc;                 /* nucleons per nucleus */
    double lum;                  /* white dwarf luminosity */
    double c = 2.9979e10;        /* speed of light */
    double g = 6.67e-8;          /* Newtonian gravitation const. */
    double t1,t2,t3;             /* temp variables */
    int jm = 0;                  /* loop counter for integration */
    int trace;                   /* trace flag */
    int startflag = 1;           /* First loop flag. do inits */
    double binding;              /* binding_energy[r]     */
    double momenti;              /* moment_of_inertia[r]  */
    double vfac;                 /* azimuthal geometry distortion */
    double delm;                 /* dm[r] mass step in integration */
    double mr;                   /* m[r] mass at r, outer sum integral */
    double mr_in;                /* m[r] mass at r, innter sum integral */
    double dvol;                  /* dv[r] differential shell volume */
    double dvol_1;                /* previous shell volume for inner sum */
    double rv;                    /* r + [dr/2.0] avg central radius */
    double rn0, rne0;             /* shell inner radius */
    double rn, rne;               /* shell outer radius */
    double dr, dre;               /* shell thickness */
    double expecr;                /* expectation value of r over theta */
    double scale;                 /* delp/equitorial_radius */
    double omega;                 /* rotational angular velocity */
    double ro0, ro;               /* density[0], density[r] */
    double eavg;                  /* avg thermal energy */
    double eth;                   /* thermal_energy per unit vol at r */
    double thermal;               /* total thermal energy to r */
    double p0, p, delp;           /* p[0], p[r], dp[r] */
    double epsilon = 1.0;         /* pressure threshold at surface */
    double mtot;                  /* mass[r] including thermal energy */
    double mtot_in;               /* lower sum mass[r] + thermal mass */
    double rotot;                 /* density[r] including thermal energy */
    double mx;                    /* mass of thermal energy to r */
    double relfac;                /* square root of metric tensor g00 */
    double eos(double pr);                 /* equation of state */
    double inveos(double pr);              /* inverse equation of state */
    double cuberoot(double v);            /* cube root */
    double seventhroot(double v);         /* seventh root */
    double set_deltar(double ro);          /* adaptive step size set routine */
    double thermen(double ro, double p); /* energy per unit volume */
    int pflag;			/* pressure < epsilon on exit = 1 */

    pflag = 0;			/* pressure < epsilon on exit sets to 1 */
    trace = 1;                /* execution trace enable = 1. Show steps. */
    omega = rotation[m];      /* set angular velocity */
    ro0 = density[m];         /* set central density */
    ro = ro0;                 /* set density[r] */
    znuc = dwarfz[m];         /* nuclear charge */
    anuc = dwarfa[m];         /* compute central temperature limit */
    tc[m] = ( (hbar*c)/(kb*cuberoot(anuc*mproton)) )*cuberoot(ro0);
    deltar = set_deltar(ro);  /* set adaptive step size */
    rn = deltar;
    dr = deltar;
    rne = deltar;                /* reinit rne later */
    dre = deltar;                /* reinit dre later */
    expecr = (2.0/3.14159)*((rn + rne)/2.0) ;
    p0 = eos(density[m]);        /* set central pressure */
    p = p0;                      /* set p[r] */
    epsilon = 1.0;               /* set threshold for cutoff */

    dvol = ((4.0/3.0)*3.14159)*(rn*rne*rne);
    dvol_1 = dvol;
    thermal = thermen(ro,p)*dvol;
    mr = dvol*ro;
    mr_in = 0.0;
    mtot = mr + (thermal/(c*c));
    mtot_in = mr_in + (thermal/(c*c));

    if(calstruc[m] == 1 && openflag == 0){ /* stellar structure file setup */
	 fp2 = fopen("stellara.txt","w");
	 openflag = 1;
    }
    if(calstruc[m] == 1){
	 fprintf(fp2,"\n\nGeneral relativistic structure.  ");
	 if(condition[m] == 8)
	    fprintf(fp2,"Neutron star. ");
	 fprintf(fp2,"Central density = %e  \n",ro);
	 fprintf(fp2,"Cond.code = %d  ",condition[m]);
	 fprintf(fp2,"List element = %d\n\n",m);
	 fprintf(fp2," Density, R/Rsolar, M/Msolar, pressure, sqrt[g00]\n\n");
    }
    vfac = 1.0 - (omega*expecr)*(omega*expecr)/(c*c);
    if(vfac >= 0.0)
       vfac = sqrt(vfac);
    else vfac = 0.0;
    binding = (3.0/5.0)*(g*mtot*mtot)/rn;
    momenti = (2.0/5.0)*mtot*rn*rne;

    /*                                                           */
    /* Principal loop -  loop limit puts ceiling on calculation. */
    /*                   INTEGRATE TO THE SURFACE, UNTIL P = 0   */

    jm = 0;
    while( p > epsilon && jm < looplim ){
       mx = thermal/(c*c);
       relfac = 1.0 - ( ((2.0*g)/(c*c))*((mr + mx)/((rn + rne)/2.0)) );
       if(relfac >= 0.0)
	  relfac = sqrt(relfac);
       else relfac = 0.0;
       dvol = 4.0*(3.14159)*rn*rne*((dr + dre)/2.0)/(relfac*vfac);
       thermal = thermal + thermen(ro,p)*dvol;
       mtot = mr + (thermal/(c*c));
       mtot_in = mr_in + (thermal/(c*c));
       rotot = ro + (thermen(ro,p)/(c*c));
       delp = ((g*mtot*rotot)/(rn*rn*relfac*vfac))*deltar;
       if(rotate == 1){
	   scale = (g*mtot*rotot/(rn*rn*relfac*vfac)) - (omega*omega*rn);
	   if(scale <= 0.0){
	      flyapart[m] = 1;
	      delp = 0.0;
	      p = 0.0;
	      ro = 0.0;
	   }
	   if(scale > 0.0 && ro > 0.0)
	      dre = delp/(scale);
	   if(startflag == 1){  /* special initializations */
	      vfac = 1.0 - ((omega*expecr)*(omega*expecr)/(c*c));
              if(vfac >= 0.0)
		 vfac = sqrt(vfac);
              else vfac = 0.0;
	      rne = dre;
	      dvol = (4.0/3.0)*(3.14159)*(rn*rne*rne)/(relfac*vfac);
	      dvol_1 = dvol;
	      mr = dvol*ro;
	      mr_in = 0.0;
	      mtot = mr + (thermal/(c*c));
	      mtot_in = mr_in + (thermal/(c*c));
	      startflag = 0;
	   }
       }
       if(rne < 0.0)  /* handle no-rotations case */
	   rne = rn;
       p = p - delp;
       if( p > epsilon ){
       		pflag = 0;
	    ro = inveos(p);
       }
       if( p <= epsilon){
	    ro = 0.0;
	    p = 0.0;
            pflag = 1;
       }
       rn0 = rn;                /* save previous step radius */
       rne0 = rne;              /* or shell inner radius     */
       rn = rn + dr;            /* find shell outer radius   */
       rne = rne + dre;         /* shell outer radius        */
       rv = ( (rn0 + (dr/2.0))  +  (rne0 + (dre/2.0)) )/2.0;
       expecr = (2.0/3.14159)*(( (rn0 + dr/2.0) + (rne0 + dre/2.0) )/2.0);

       vfac = 1.0 - ( (omega*expecr)*(omega*expecr)/(c*c) );
       if(vfac >= 0.0)
	  vfac = sqrt(vfac);
       else vfac = 0.0;
       if(rotate == 0){
	   dvol = (4.0/3.0)*3.14159*(rn0*rn0)*deltar/relfac;
	   mr = mr + (dvol*ro);           /* outer sum */
	   mr_in = mr_in + (dvol_1*ro);   /* inner sum */
	   binding = binding + (g*mtot*delm/rv);
	   momenti = momenti + (2.0/3.0)*delm*rn0*rn0;
       }
       if(rotate == 1){ /* if rotating, use GR ellipsoid shell volume */
	dvol = (4.0/3.0)*(3.14159);
	dvol = dvol*((rne0 + dre)*(rne0 + dre)*(rn0 + dr) - (rne0*rne0*rn0));
        if(vfac >= 0.0)        // look for divide by zero error.
	    dvol = dvol/(relfac*vfac);
        else dvol = 1.0e20; // simulate infinity
	mr = mr + (dvol*ro);           /* outer sum integral */
	mr_in = mr_in + (ro*dvol_1);   /* inner sum integral */
	binding = binding + (g*mtot*delm/rv);
	momenti = momenti + (2.0/3.0)*delm*rv*rv;
       }
       if(trace == 1){
	  printf("\nmodel[%d] of %d loop[%d], ro[0] = %e\n",m,j,jm,ro0);
	  printf("g[0,0] =  %e\n",relfac*relfac);
	  printf("mr = %e gr,   mr = %e solar masses\n",mr, mr/msolar);
	  printf("mr[lower bound]  = %e solar masses\n",mr_in/msolar);
	  printf("dvol = %e, m[r]+[thermal/c*c] = %e\n",dvol,mtot);
	  printf("rotot = %e, rp = %e km\n",rotot,delp,rn0/100000.0);
	  printf("p = %e, dp = %e\n",p, delp);
	  printf("polar axis = %e km  ", rn/100000.0);
	  printf(" equitorial axis r = %e km\n",rne/100000.0);
	  printf("moment_of_inertia[r] = %e\n",momenti);
	  printf("thermal_energy[r] = %e ergs\n",thermal);
	  printf("binding_energy[r] = %e ergs\n\n",binding);
       }
       if(calstruc[m] == 1){      /* OUTPUT TO STRUCTURE FILE */
	  fprintf(fp2,"%e  %e  %e  %e",ro, rn0/rsolar, mtot/msolar, p);
	  fprintf(fp2,"  %e\n",relfac);
       }
       dvol_1 = dvol;
       jm++;
    }

    printf("\nPrepare summary calculations.\n");
    mtot = mr + (thermal/(c*c));       /* prepare summary */
    mtot_in = mr_in + (thermal/(c*c));
    drx[m] = dr;		/* save step differential dr in cm */
    radiusratio[m] = rn/rsolar;
    majorratio[m] = rne/rsolar;
    massratio[m] = mtot/msolar;     /* upper sum integral for mass */
    massinner[m] = mtot_in/msolar;  /* inner sum integral mass - lower lim */
    if( rn >  0.0)
       redshift[m] = (g*mtot/rn)/(c*c);
    else redshift[m] = 1.0e20;  // simulate infinity if divide by zero happens.
    thermal_energy[m] = thermal;
    binding_energy[m] = binding;
    moment[m] = momenti;
    steps[m] = jm;
    pflagx[m] = pflag;	/* pressure < epsilon at exit sets to 1, else 0. */

    lumin[m] = 0.0;
    ct[m] = 0.0;
    fixed = 0;
    if(looplim > 100 && fixed == 1){  /* Bowers&Deeming luminosity model */
      printf("Compute white dwarf luminosity.\n");
      t1 = tc[m];
      if( t1 >= 0.0)        /* compute white dwarf luminosity */
	  t1 = sqrt(t1);
      else t1 = 0.0;
      t2 = t1*t1*t1*t1*t1*t1*t1;
      lumin[m] = (5.7e25)*(10.0/znuc)*(anuc/4.0)*(mr/msolar)*t2;
      lum = lumin[m];
      printf("white dwarf luminosity = %e ergs\n",lum);
      if( lum/(4.0*3.14159*sigma*rn*rn) > 0.0 )
	   teff[m] = sqrt(sqrt(lum/(4.0*3.14159*sigma*rn*rn)));
      else teff[m] = 0.0;
      lumin[m] = lum/lsolar;

      printf("Compute white dwarf cooling time.\n");
      if(lum > 0.0)
	  t1 = seventhroot( (mr/msolar)*(lsolar/lum) );  /* cal cooling time */
      else t1 = 1.0e20; // simulate infinity if divide by zero happens.
      t1 = t1*t1*t1*t1*t1;
      t2 = seventhroot( ((4.0/anuc)*(1.0/10.0)*znuc) );
      ct[m] = (8.9e7)*(t1*t2*t2/anuc);
    }

    printf("End prepare summary calculations.\n");
    if(m == j - 1 && openflag == 1){
	fprintf(fp2,"\n\n");
	status = fclose(fp2);
	printf("\nClose STELLARA.TXT\n");
    }
}
/*************************************************
*
*  inveos - Equation of state of a Fermi gas inverse.
*           density = inveos(pressure)
*           Uses a binary search procedure with (electrons)
*           x_upper = (8.516735e-6)*sqrt(sqrt(pressure)).
*           Finds x such that (electrons)
*           P = (1.9107636e22)*f(x).
*           Then   density = (1.9648461e6)*(x*x*x).
*           Also computes inverse eos for neutron gas.
*
*  in: pressure, condition[m], ro00 current model starting density.
* out: density
*
****************************************************/
double inveos( double pr)
{
    int tstflag, n, nnn;
    double xupper, xlower, xmid, eospr, ro, del, den2, v1, v2;
    double roupper, rolower, romid, epsilon;

    tstflag = 0;    /* enable test trace = 1 */
    epsilon = pr/1.0e7;
    n = 0;
    if(condition[m] < 50){   /* Fermi gas of electrons or similar */
       if(pr >= 0.0)
	   v1 = sqrt(sqrt(pr));
       else v1 = 0.0;
       xupper = 2.0*(8.516735e-6)*v1;
       if(v1 >= 0.0)
	  v2 = sqrt(v1);
       else v2 = 0.0;
       if(v2 >= 0.0)
	 xlower = (8.516735e-6)*sqrt(v2);
       else xlower = 0.0;
       roupper = 1.5*(1.9648461e6)*(xupper*xupper*xupper);
       rolower = (1.0e6)*xlower*xlower*xlower;
       roupper = 10.0*roupper;
       rolower = 0.1*rolower;  /* stretch bounds for special EOS */
    }
    if(condition[m] == 8){   /* exception: pure Fermi gas of neutrons  */
       if(pr >= 0.0)
	  v1 = sqrt(sqrt(pr));
       else v1 = 0.0;
       xupper = 2.0*(1.852349e-9)*v1;
       if(v1 >= 0.0)
	    v2 = sqrt(v1);
       else v2 = 0.0;
       xlower = (1.852349e-9)*v2;
       roupper = 1.5*(xupper*xupper*xupper)/(1.6363627e-16);
       rolower = (xlower*xlower*xlower)/1.6363627e-16;
    }
    if(condition[m] == 15 || condition[m] == 16){  /* D-quark sea */
	roupper = 1.5*ro00;
        rolower = 0.0;
    }
    if(condition[m] == 17){  /* relativistic neutron sea */
	roupper = 1.5*ro00;
	rolower = 0.0;
    }
    if(tstflag == 1){
       printf("ro_upper0 = %e, ro_lower0 = %e\n", roupper, rolower);
    }

    del = pr;
    while( del > epsilon && n < 45){             /* binary search loop */
       romid = rolower + ((roupper - rolower)/2.0);
       eospr = eos(romid);
       if(eospr < pr)
	  rolower = romid;
       if(eospr > pr)
	  roupper = romid;
       del = eospr - pr;
       if(del < 0.0)
	  del = -del;
       if(tstflag == 1){
	  printf("n = %d\n",n);
	  printf("pr = %e, eospr = %e, del = %e\n", pr, eospr, del);
	  printf("density_midpoint = %e \n", romid);
       }
       n++;
     }
     if(tstflag == 1){
	 printf("\n\nINVEOS_density = %e\n",romid);
     }
     return(romid);
}
/******************************************
*
*  eos - equation of state of a Fermi gas.
*        Pressure = eos(density). Use the
*        relativistic Thomas-Fermi electrostatic
*        correction factor for Fe56 core on
*        select condtion codes.
*        Works for electron degeneracy pressure
*        and neutron degeneracy pressure.
*
*  in:  density,
*       novas == 1 => znuc = 26, else znuc set for
*                              white dwarfs.
*  out: pressure
*
*******************************************/
double eos(double ro)
{
     int znuc;
     double zznuc;
     double cuber,x,p;
     double fro;
     double bowersdelta;     /* screening 1: Bowers & Deeming delta P */
     double wignerscaler;    /* screening 2: Alternative - Wigner-Seitz */

     if(novas == 1){
	  znuc = 26;
	  dwarfz[m] = 26;
     }
     if(novas == 0){           /* set EOS Z for white dwarf model no. m */
	  znuc = dwarfz[m];
     }
     zznuc = znuc;                   /* convert Z to floating point */

     cuber = cuberoot(zznuc);        /* compute wigner-seitz correction */
     wignerscaler = (1.0 - ((0.004564)*cuber*cuber) );

     if(condition[m] < 8){           /* Fermi gas of electrons */
	x = cuberoot(ro);
	x = x/(1.2524955e2);
	p = (1.9107636e22)*f2(x);    /* alternate representation */

	if(condition[m] == 2 || condition[m] == 3){
	   bowersdelta = bowers(ro);
	   p = p + bowersdelta;  /* alternative is p = p*wignerscaler */
	}
	if(condition[m] == 4 || condition[m] == 5){
	   p = eosbb(ro);
	}
	if(condition[m] == 6 || condition[m] == 7){
	   p = eosbb(ro);
	   bowersdelta = bowers(ro);
	   p = p + bowersdelta;  /* alternative is p = p*wignerscaler */
	}
     }
     if(condition[m] == 8){             /* Fermi gas of neutrons */
	x = (5.469654e-6)*cuberoot(ro);
	p = (1.351852e34)*f2(x);
     }
     if(condition[m] == 9){
	p = eoshw(ro);
     }
     if(condition[m] == 10){
       p = eoshw(ro);
     }
     if(condition[m] == 11){
       p = eoshw(ro);
       bowersdelta = bowers(ro);
       p = p + bowersdelta;      /* alternative is p = p*wignerscaler */
     }
     if(condition[m] == 12){
	p = eosbb(ro); /* baym-bethe with zeldovich quark EOS in limit */
     }
     if(condition[m] == 13){  /* Bowers&Deeming non-rel degen. neutron gas */
	fro = cuberoot(ro);
	p = (5.4e9)*fro*fro*fro*fro*fro;
     }
     if(condition[m] == 14){ /* Bowers&Deeming non-rel d. n. gas w quark */
	fro = cuberoot(ro);
	p = (5.4e9)*fro*fro*fro*fro*fro;
	if(ro > 1.44e16)
	  p = (2.9958e20)*ro;
     }
     if(condition[m] == 15){ /* unscreened D-Quark sea */
        x = cuberoot(ro);
	x = x*(1.0/alpha)*3.21714e-6;
	p = alpha*alpha*alpha*alpha; /* D-quark_mass = alpha x neutron */ 
	p = p*(2.191e35)*f2(x);    /* D-Quark gas. */

     }
     if(condition[m] == 16){ /* screened D-quark sea */
	x = cuberoot(ro);
	x = x*(1.0/alpha)*3.21714e-6;
	p = alpha*alpha*alpha*alpha; /* D-quark_mass = alpha x neutron */ 
	p = p*(2.191e35)*f2(x);    /* D-Quark gas. */
	p = 0.996544*p; /*electrostatic screen due to +2/3e U-quark, */
			/* see p.32, eqn2.4.13, Shapiro & Teukolsky, */
			/* BLACK HOLES, WHITE DWARFS,& NEUTRON STARS. */
     }
     if(condition[m] == 17){ /* relativitistic neutron sea */
	x = cuberoot(ro);	/* same as D-quark sea, but alpha=1, & */
	x = x*(3.21714e-6)*(0.7937); /* 1 neutron per neutron, */
	p = 2.191e35*f2(x);          /* not 2 D-quarks neutron */
     }     
     if( ro > 1.44e16 && quarks == 1){   /* Zeldovich quark EOS */
	 p = (2.9958e20)*ro;             /* is conditional */
     }
     return(p);
}
/********************************************************
*
*   bowers - compute bowers & deeming pressure correction
*            for electrostatic screening.
*
*********************************************************/
double bowers(double ro)
{
     double z;            /* nuclear charge */
     double anuc;         /* nucleon number */
     double b1,b2,b3,b4;  /* factors */
     double deltap;       /* screening pressure differential */
     double cuberoot(double ro);

     b1 = cuberoot(ro);
     b1 = b1*b1*b1*b1;
     z = dwarfz[m];
     anuc = dwarfa[m];
     b2 = cuberoot(anuc);
     b2 = b2*b2*b2*b2;
     b3 = cuberoot(z);
     b4 = ((1.0 - b3)*z*z)/b2;
     deltap = (1.94e12)*b4*b1;
     return(deltap);
}
/***********************************************
*
*  eqn of state: P = (m**4c**5/3h**3)*f(x)
*
*  f(x) = Integral[g(x)dx]*8*pi, g(x) = x**4/sqrt(1.0+x*x).
*          0 to x.
*         This is the integral form of f(x).
*         The analytic form is f2(x).
*
************************************************/
double f( double x)
{
    double dN;
    double delx,dx;
    double sum;
    double da,db,jmd;
    int N,jm;

    N = 100;
    dN = 100.0;
    delx = x/dN;
    sum = 0.0;
    jmd = 0.0;
    for(jm=0; jm < N; jm++){
      da = jmd*delx;
      db = (jmd + 1.0)*delx;
      sum = sum + (( gg( da ) + gg( db ) )/2.0)*delx;
      jmd = jmd + 1.0;
    }
    sum = sum*(8.0*3.14159);
    return(sum);
}
/*****************************************
*
*  gg(y) - y**4/(1 + y**2) is the kernel of the
*         integral representation of the
*         equation of state.
*
******************************************/
double gg( double y)
{
    double q;

    if( 1.0 + (y*y) >= 0.0 )
	q = (y*y*y*y)/sqrt( 1.0 + (y*y) );
    else{
       q = 1.0e50;  // simulate infinity
    }
    return(q);

}
/*******************************************
*
*   eoshw(ro) - Harrison-Wheeler eqn of state.
*               Piecewise linear approx. in log-log.
*               Calculates pressure from density.
*
********************************************/
double eoshw(double ro)
{
      double log_p, log_ro;
//      double log10();    now global Nov.30, 1999
//      double exp();
      double a1,b1,a2,b2,a3,b3,a4,b4,p;

      a1 = 1.5909091;
      b1 = 10.431818;
      a2 = 0.5556;
      b2 = 23.166667;
      a3 = 1.02273;
      b3 = 17.840878;
      a4 = 1.818182;
      b4 = 12.27273;

      log_ro = log10(ro);
      if(log_ro > 12.3){
	 log_p = (a1*log_ro) + b1;
	 p = exp(2.3025851*log_p);
      }
      if(log_ro <= 12.3 && log_ro > 11.4){
	 log_p = (a2*log_ro) + b2;
	 p = exp(2.3025851*log_p);
      }
      if(log_ro <= 11.4 && log_ro > 7.0){
	 log_p = (a3*log_ro) + b3;
	 p = exp(2.3025851*log_p);
      }
      if(log_ro <= 7.0){
	 log_p = (a4*log_ro) + b4;
	 p = exp(2.3025851*log_p);
      }
      return(p);
}
/*****************************************
*
*  thermen(ro,p) - Compute the thermal energy
*                per unit volume using the
*                electron gas model for the pressure.
*                For ro > 1.0e11, use (3.0/2.0)*p.
*                This is at least excellent for white dwarfs.
*                The higher density estimate is for a non-relativistic
*                neutron gas.
*
******************************************/
double thermen(double ro, double p)
{
     double x, en;
// formerly     double chi();           now global.
// formerly     double cuberoot();

     if(ro < 1.0e11){  /* electron gas model for white dwarfs */
	x = (5.469954e-6)*cuberoot(ro);
	en = 1.4218e25*chi(x);
     }
     if(ro >= 1.0e11)  /* non-relativistic neutron gas model for e */
	en = (3.0/2.0)*p;
     return(en);
}
/*************************************
*
*  chi - Energy function for an electron gas.
*        (See Shapiro, pg 25.)
*
**************************************/
double chi(double x)
{
    double v;

    v = (1/(8.0*3.14159*3.14159));
    if( 1.0 + x*x >= 0.0)
      v = v*(x*sqrt((1.0 + x*x))*(1.0 + 2.0*x*x) - log(x + sqrt(1.0 + x*x)));
    else v = v*( - log(x) );
    if(v < 0.0)
	v = 0.0;
    return(v);
}
/*****************************************
*
*   eosbb - Baym-Bethe with Quark limit.
*
******************************************/
double eosbb(double ro)
{
      double x, p;
      double val;

/*                        This linearized [in the log-log plane]   */
/*                        representation may be improved.          */

      if( ro > 1.44e16 ){
	  p = (2.9958e20)*ro;
	  return(p);
      }
      if( ro <= 1.44e16 && ro > 1.0e15){
	  val = 3.07158*log10(ro) + 34.746603;
	  p = exp( val );
	  return(p);
      }
      if( ro <= 1.0e15 && ro > 1.0e13){
	  val = 3.9143947*log10(ro) + 19.571973;
	  p = exp( val );
	  return(p);
      }
      if( ro <= 1.0e13 && ro > 1.0e12){
	  val = 1.3815511*log10(ro) + 52.49894;
	  p = exp( val );
	  return(p);
      }
      if( ro > 1.0e11 && ro <= 1.0e12 ){
	  val = 2.3025851*log10(ro) + 41.446532;
	  p = exp( val );
	  return(p);
      }
      if( ro > 1.0e6 && ro <= 1.0e11){
	  val = 3.1775674*log10(ro) + 31.821726;
	  p = exp( val);
	  return(p);
      }
      if( ro <= 1.0e6 ){  /* electron gas w/o Wigner-Seitz corr. */
	 x = cuberoot(ro);
	 x = x/(1.2524955e2);
	 p = (1.9107636e22)*f2(x);
	 return(p);
      }
      printf("RANGE ERROR in Baym-Bethe eqn of state.\n");
      p = 0.0;
      return(p);
}
/******************************************
*
*  nthroot(x) - Cal the nth root of x.
*
*
*******************************************/
double nthroot(double xx, int nn)
{
      int vi, i, doitagain;
      double vdelta,diff,testval,midpnt,lower,upper;
      double xu,epsilon,sign;

      epsilon = 0.00000001;
      xu = xx;
      sign = 1.0;
      if( xu < 0.0){
	 sign = -1.0;
	 xu = -xu;
      }
      lower = 0.0;
      if(xu >= 1.0)
	  upper = 1.5*sqrt(xu);
      else
	  upper = 1.0;
      doitagain = 1;

      i = 0;                           /* binary search loop */
      while( i < 200 && doitagain == 1){
	 midpnt = lower + ((upper - lower)/2.0);
	 testval = 1.0;
	 for(vi = 0; vi < nn; vi ++)
	    testval = midpnt*testval;
	 diff = testval - xu;
	 vdelta = diff;
	 if( vdelta < 0.0)
	    vdelta =  0.0 - vdelta;
	 if( vdelta > epsilon)
	    doitagain = 1;
	 if( vdelta <= epsilon)
	    doitagain = 0;
	 if( diff < 0.0 )
	    lower = midpnt;
	 if( diff > 0.0 )
	    upper = midpnt;
	 if( diff = 0.0 )
	    doitagain = 0;
	 i = i + 1;
      }
      return(sign*midpnt);
}
/*******************************************
*
*   cuberoot(v) - Compute the cube root of v.
*
********************************************/
double cuberoot(double v)
{
      double zout;

      zout = nthroot(v,3);
      return(zout);
}
/********************************************
*
*   seventhroot(v) - Compute the seventh root of v.
*
*
*********************************************/
double seventhroot(double v)
{
       double zout;

       zout = nthroot(v,7);
       return(zout);
}
/********************************************
*
*  f2 - Alternate f(x) function for the equation
*       of state using a closed-form analytic
*       expression for the relativistic integral.
*       Relies on binary search method for inverse
*       sinh.
*
*********************************************/
double f2(double z)
{
    double g2;
    int trace;

    trace = 0;
    if(trace == 1){
       printf("function f2(z), z = %e\n",z);
       printf("invsinh(z) = %e\n\n", invsinh(z));
    }
    if( 1.0 + (z*z) > 0.0)
	g2 = z*((2.0*z*z) - 3.0)*sqrt( (z*z) + 1.0 );
    else g2 = 0.0;
    g2 = g2 + (3.0*invsinh(z));
    g2 = 3.14159*g2;
    return(g2);
}
/***************************************************
*
*  invsinh() - Arcsinh(x) function. Uses binary search.
*
****************************************************/
double invsinh(double q)
{
     double epsilon = 0.0000001; /* precision threshold */
     double del;                 /* sinh search difference */
     double tu, tl, tm;          /* upper limit, lower limit, midpoint */
     int nu;                     /* loop counter  & cutoff */

     del = 2.0*epsilon;
     if(q > 1.0){
	   tu = (3.0*log(2.0*q)) + 1.0;
     }
     if(q <= 1.0){
	   tu = 10.0;
     }
     tl = 0.0;
     nu = 200;
     while(del > epsilon && nu > 0){
	  tm = tl + (0.5*(tu - tl));
	  del = q - sinh(tm);
	  if(del > 0.0)
	     tl = tm;
	  if(del < 0.0){
	      tu = tm;
	      del = -del;
	  }
      nu--;
      }
      return(tm);
}
/********************************************
*
*  set_deltar - Set step size according to
*               density adaptively.
*
*  in: step_size = step divisor for extra precision.
*
*********************************************/
double set_deltar(double ro)
{
      double stepmag;    /* floating step divisor */
      double delta;      /* final step magnitude, cm */

      if(ro < 1.0e9 )
	  delta = 10.0e5;               /* 10 kilometers */
      if(ro <= 1.0e10 && ro >= 1.0e9)
	  delta = 5.0e5;                /* 5 kilometers  */
      if(ro <= 1.0e12 && ro >= 1.0e10 )
	  delta = 5.0e5;
      if(ro <= 1.0e13 && ro >= 1.0e12)
	  delta = 2.0e5;
      if(ro <= 1.0e14 && ro >= 1.0e13)
	  delta = 2.0e5;
      if(ro > 1.0e14 && ro <= 1.0e15)
	  delta = 0.5e5;
      if(ro > 1.0e15 && ro <= 1.0e17)
	  delta = 0.10e5;
      if(ro > 1.0e17 && ro <= 1.0e19)
	  delta = 0.10e5;
      if(ro > 1.0e19)
	  delta = 0.05e5;
      if(condition[m] == 8){  /* use smaller steps on pure neutron stars */
	  delta = delta/10.0;
      }

      stepmag = step_size;         /* optional step divisor 1,2,3.. */
      delta = delta/stepmag;
      return(delta);
}
/**************************************************
*
*    save - Save restart file, including all input
*           and output variables. Invoked before
*           each model generation.
*
***************************************************/
void save(void)
{
       int max;
       int status;
       FILE *fp;

       printf("BUILDING RESTART SAVE FILE at model %d of %d.\n",m,jsave);
       printf("DO NOT INTERRUPT UNTIL RESTART FILE CONSTRUCTED.\n");

       fp = fopen("restart.txt", "w");

       fprintf(fp,"%d\n",looplim);
       fprintf(fp,"%d\n",novas);
       fprintf(fp,"%d\n",znuc);
       fprintf(fp,"%d\n",m);
       fprintf(fp,"%d\n",jsave);
       fprintf(fp,"%d\n",rotate);
       fprintf(fp,"%d\n",greens);
       fprintf(fp,"%d\n",quarks);
       fprintf(fp,"%d\n",step_size);
       fprintf(fp, "%e\n",deltar);
       fprintf(fp, "%d\n",openflag);
       fprintf(fp, "%d\n",video);
       fprintf(fp, "%d\n",model0);

       for(max = 0; max < 100; max++){
	   fprintf(fp,"%e\n",density[max]);
	   fprintf(fp,"%e\n",rotation[max]);
	   fprintf(fp,"%d\n",condition[max]);
	   fprintf(fp,"%d\n",calstruc[max]);
	   fprintf(fp,"%d\n",dwarfz[max]);
	   fprintf(fp,"%e\n",dwarfa[max]);
	   fprintf(fp,"%e\n",massratio[max]);
	   fprintf(fp,"%e\n",massinner[max]);
	   fprintf(fp,"%e\n",radiusratio[max]);
	   fprintf(fp,"%e\n",majorratio[max]);
	   fprintf(fp,"%d\n",steps[max]);
	   fprintf(fp,"%e\n",binding_energy[max]);
	   fprintf(fp,"%e\n",thermal_energy[max]);
	   fprintf(fp,"%e\n",moment[max]);
	   fprintf(fp,"%e\n",redshift[max]);
	   fprintf(fp,"%e\n",ct[max]);
	   fprintf(fp,"%e\n",lumin[max]);
	   fprintf(fp,"%e\n",teff[max]);
	   fprintf(fp,"%e\n",tc[max]);
	   fprintf(fp,"%d\n",flyapart[max]);
       }

       status = fclose(fp);
       printf("RESTART FILE CONSTRUCTED for model %d of %d.\n",m,j);
}
/************************************************
*
*   restore - If user desires, restart from save file.
*             Use RESTART.TXT.
*
************************************************/
void restore(void)
{
       double ev;
       void processlist(void);
       int dv;
       int max;
       int status;
       int rsflag;
       FILE *fp;

       setdef();
       fp = fopen("restart.txt","r");

       printf("RESTORING MACHINE STATE FROM FILE RESTART.TXT...\n");
       fscanf(fp,"%d\n",&looplim); printf("looplim = %d\n",looplim);
       fscanf(fp,"%d\n",&novas);   printf("novas = %d\n",novas);
       fscanf(fp,"%d\n",&znuc);    printf("znuc = %d\n",znuc);
       fscanf(fp,"%d\n",&m);       printf("model number = %d\n",m);
       fscanf(fp,"%d\n",&j);       printf("total models = %d\n",j);
       fscanf(fp,"%d\n",&rotate);
       fscanf(fp,"%d\n",&greens);
       fscanf(fp,"%d\n",&quarks);
	printf("rotate = %d, grns = %d, quarks = %d\n",rotate,greens,quarks);
       fscanf(fp,"%d\n",&step_size); printf("step_size = %d\n",step_size);
       fscanf(fp,"%lf\n",&deltar);
       fscanf(fp,"%d\n",&openflag);
       fscanf(fp,"%d\n",&video);  printf("deltar = %e \n", deltar);
       fscanf(fp,"%d\n",&model0); printf("model0 = %d\n",model0);

	 printf("openflag = %d, video = %d\n",openflag,video);

       for(max = 0; max < 100; max++){
	   fscanf(fp,"%lf\n",&ev);
	       density[max] = ev;
	   fscanf(fp,"%lf\n",&ev);
	       rotation[max] = ev;
	   fscanf(fp,"%d\n", &dv);
	       condition[max] = dv;
	   fscanf(fp,"%d\n", &dv);
	       calstruc[max] = dv;
	   fscanf(fp,"%d\n",&dv);
	       dwarfz[max] = dv;
	   fscanf(fp,"%lf\n",&ev);
	       dwarfa[max] = ev;
	   fscanf(fp,"%lf\n",&ev);
	       massratio[max] = ev;
	   fscanf(fp,"%lf\n",&ev);
	       massinner[max] = ev;
	   fscanf(fp,"%lf\n",&ev);
	       radiusratio[max] = ev;
	   fscanf(fp,"%lf\n",&ev);
	       majorratio[max] = ev;
	   fscanf(fp,"%d\n", &dv);
	       steps[max] = dv;
	   fscanf(fp,"%lf\n", &ev);
	       binding_energy[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
	       thermal_energy[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		moment[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		redshift[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		ct[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		lumin[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		teff[max] = ev;
	   fscanf(fp,"%lf\n", &ev);
		tc[max] = ev;
	   fscanf(fp,"%d\n", &dv);
		flyapart[max] = dv;
       }
       jsave = j;

       status = fclose(fp);

       printf("densities = ");
	  for(max = 0; max < 4; max++)
	      printf("%e,  ", density[max]);
	  printf("...\n");

       printf("rotation = ");
	  for(max = 0; max < 4; max++)
	      printf("%e, ",rotation[max]);
	  printf("...\n");

       printf("MACHINE STATE RESTORED.  RESTARTING:\n\n");

       fp2 = fopen("stellara.txt","a");
       openflag = 1;

       if(model0 == 0)  /* if not interrupted in model 0, increment */
	  m++;
       processlist();
       file_list();
       printf("Series End.  ");

       printf("PROGRAM WAS RESTARTED.   EXIT.\n");

       exit(0);

}
/************************************************************/
//
//  STAR SKETCH - Eddington Standard Model Stars.
//
//
/************************************************************/
/**************************************************************
*
*   polytrope() - Do quick sketch of polytropic star from
* 		  clues about its structure and so forth.
*
****************************************************************/
void polytrope(void)
{
	int pselect = 1;
	int chemsw = 1;         // chemistry switch
	int basicsw = 6;       // basic property selection switch.
	int pauseflag;        // routine pauses until pauseflag loaded.
	double xhydro;		// hydrogen abundance by weight,
	double yhelium;		// helium abundance by weight,
	double zheavy;		// heavy element abundance by weight.
	double muinv;		// 1/mu = 2X + (3/4)Y + 0.5(Z).
	double mratio; 		// M/Msun.
	double rratio;		// R/Rsun.
	double avgdensity;	// average density of the star.
	double rstar;		// stellar radius in meters.
	double densitycentral;  // central density in kg.
	double msun = 1.99e30;  // mass of the sun in kg.
	double rsun = 6.96e8;   // radius of the sun in meters.
	double factor;		// factor in pressure calc.
	double pressurecentral; // central pressure
	double mstar;		// mass of the star
	double mh = 1.6748e-27;	// mass of a hydrogen atom in kg.
	double k = 1.3805e-23;  // Boltzmann constant.
        double sigma = 5.6697e-8; // Stefan-Boltzman constant in W/(m**2 K**4).
	double a = 7.564895e-16;  // radiation constant = 4*sigma/c.
	double fbeta;		// (1-b)/b**4 = (P**3/ro**4)*[(mu*mh/k)**4]*(a/3).
	double beta;            //  b.
	double pgas;		// central gas pressure.
	double pradiation;      // central radiation pressure.
	double pratio;          // pradiation/pgas;
	double tc;		// central temperature
//                                 tc=(mu*mh/(k*density_c))*beta*Ptotal_c
	double omega;		// stellar binding energy in Joules.
	double findbeta(double beta);
	void distancecal(void);
	void luminosity(void);
	void stellarmass(void);
	void stellarradius(void);
	void tsurf(void);
	void sublist(void);

	
	pselect = 1;
	chemsw = 1;
        basicsw = 6;
	while(pselect != 0){
	  printf("\nSTAR SKETCH - Eddington Standard Model Stars n=3, gamma=4/3.\n");
	  printf("NOTE: Distances are rather directly measured via the parallax.\n");
	  printf("If the magnitude m of the star is known and it's distance r has been\n");
	  printf("determined, L/Lsun = 10**[[4.8 - [m + 5 - 5*log[r]]/2.5] \n");
	  printf("where r is given in parsecs and 1 parsec = 3.26 light years,\n");
	  printf("since by definition  I1/I2 = 10**[[2/5][m2 - m1]], so that \n");
	  printf("[L1/L2][r2/r1]**2 = 10**[[2/5][m2 - m1]. Note at 10 pc, m_sun = 4.8.\n");
	  printf("Masses M are determined by double-star orbit measurements for double\n");
	  printf("stars, or are found from the mass-luminosity curve, where\n");
	  printf("L = B[M]**x, where  3 < x[M] < 4.  Therefore,  M/Msun = [L/Lsun]**[1/x].\n");
	  printf("From black-body radiation physics, the surface temperature of the star\n");
	  printf("is determined by it's spectral type. Then the stellar radius R is\n");
	  printf("determined from L = [4*pi*[R]**2]*(ac/4)*T**4, when L and T are known.\n");
	  printf("Thus the distance r, the luminosity ratio L/Lsun, M/Msun, and radius R\n");
	  printf("are in principle determined by simple equations from basic measurements.\n");
	  printf("BASIC STELLAR VARIABLES - with & without bolometric corrections BC[T].\n");
          printf("ENTER 0 to list sample Main Sequence star data.\n");
	  printf("      1 to find distance to star r from the parallax in seconds of arc.\n");
	  printf("      2 L/Lsun, M/Msun, R/Rsun & M from r in LY, m, and Tsurface,w.BC.\n");
	  printf("      3 L/Lsun, M/Msun, R/Rsun & M from r in LY, m, and Tsurface,w/o BC.\n");
	  printf("      4 to estimate Tsurface from the spectral type.\n");
	  printf("      5 to find just R/Rsun from the surface temp Tsurface and L/Lsun.\n");
	  printf("      6 to continue on to the standard model polytrope calculations.\n");
	  printf("      7 to abort : ");
	  scanf("%s",value);   // This
          d = atof(value);     // entry procedure is more immune to invalid inputs
          basicsw = d;      // than simple scanf().

	  if(basicsw > 7 || basicsw < 0){
	      printf("Entry Error. Defaulting to 6 = on to polytrope modeling.\n");
	      basicsw = 6;
	  }
	  if(basicsw == 0){
	      sublist();
	      printf("Enter 1 continue: ");
	      scanf("%s",value);   // This
       	      d = atof(value);     // entry procedure is more immune to invalid inputs
       	      pauseflag = d;      // than simple scanf().

          }
	  if(basicsw == 1)
	      distancecal();  // calculate the distance from the parallax in arcsec.
	  if(basicsw == 2)
	      luminosity();   // cal L/Lsun, M/Msun absolute mag. M from r[ly] and m.
	  if(basicsw == 3)
	      luminosity2();  // cal L/Lsun, M/Msun, abs mag. M, from r[pc] and m.
	  if(basicsw == 4)
	      tsurf();		// display spectral types and temps
	  if(basicsw == 5)
	      stellarradius();  // cal R/Rsun from Tsurf and L/Lsun.
	  if(basicsw == 6)
             pselect = 0;     // drop through and do next phase.
	  if(basicsw == 7){
	      printf("EXIT CHANDRA");
              exit(0);
	  }
	}
	pselect = 1;
	while(pselect != 0){
	  printf("\nPOLYTROPE SECTION - STAR SKETCH\n");
	  printf("In the Eddington Standard Model, stars are polytropes with gamma=4/3,\n");
	  printf("and the molecular weight is how the chemistry of the star enters the\n");
	  printf("model, influencing the ratio of gas pressure to radiation pressure.\n");
	  printf("ENTER\n");
          printf("  0 to list typical Main Sequence star variables.\n");
	  printf("  1 mu for M.Schwarzshild Main Seq. Chemistry, X=0.75, Y=0.23, Z=0.02.\n");
	  printf("  2 mu for J.A.Green Helium-Rich Chemistry, X= 0.63, Y=0.35, Z=0.02.\n");
	  printf("  3 mu - Specify your own chemistry X,Y,Z, hydrogen, helium, heavy.\n");
	  printf("  4 mu - Specify the molecular weight mu more precisely yourself.\n");
	  printf("  5 to skip polytropic calc & restart STAR SKETCH calc above.\n");
	  printf("  6 \n");
	  printf("  7 to abort: ");
	  scanf("%s",value);   // This
       	  d = atof(value);     // entry procedure is more immune to invalid inputs
       	  chemsw = d;      // than simple scanf().

	  if(chemsw > 7 || chemsw < 0){
             chemsw = 1;
	     printf("Entry Error. Using 1 = Schwartzschild Chemistry as default.\n");
	  }
	  xhydro = 0.0;
	  yhelium =  0.0;
	  zheavy = 0.0;
          if(chemsw !=5 && chemsw !=7 && chemsw != 6 && chemsw != 0){   // compute the molec. weight mu.
	     if(chemsw == 1){
		xhydro = 0.75;
		yhelium = 0.23;
                zheavy = 0.02;
		mu = 0.59435;
                printf("\n Molecular weight mu = %f\n",mu);
	     }
	     if(chemsw == 2){
		xhydro = 0.63;
		yhelium = 0.23;
                zheavy = 0.02;
		mu = 0.6525285;
		printf("\n Molecular weight mu = %f\n",mu);
	     }
	     if(chemsw == 3){
		printf("Enter hydrogen abundance X by weight, 0 - 1 : ");
		scanf("%s",value);   // This
      	        xhydro = atof(value);     // entry procedure is more immune to invalid inputs

		printf("Entered %f\n", xhydro);
		if(xhydro < 0.0 || xhydro > 1.0)
		   printf("Unphysical value entered.\n");
		printf("\nEnter helium abundance by weight, < 1 - X = %f :  ", 1.0 - xhydro );
		if(1.0 - xhydro < 0.0)
                   printf("Unphysical value entered.\n");
		scanf("%s",value);   // This
       		yhelium = atof(value);     // entry procedure is more immune to invalid inputs

		printf("Entered %f\n",yhelium);
		zheavy = 1.0 - xhydro - yhelium;
		printf("\nAssume heavy element abundance 1 - X - Y = %f\n\n",zheavy);
		muinv = 2.0*xhydro + (3.0/4.0)*yhelium + 0.5*zheavy;
		if(muinv < 0.0){
                   printf("Unphysical value entered.\n");
                }
		mu = 1.0/muinv;
                printf("Molecular weight mu = %f\n",mu);
	     }
	     if(chemsw == 4){
		printf("Enter the molecular weight :  ");
		scanf("%s",value);   // This
       		mu = atof(value);     // entry procedure is more immune to invalid inputs

		printf("Entered molecular weight mu = %f.\n",mu);
	     }
	     printf("\nEnter the mass ratio M/Msun for this star : ");
	     scanf("%s",value);   // This
       	     mratio = atof(value);     // entry procedure is more immune to invalid inputs

	     printf("Entered M/Msun = %f.\n", mratio);
	     printf("\nEnter the radius  R/Rsun for this star  : ");
	     scanf("%s",value);   // This
       	     rratio = atof(value);     // entry procedure is more immune to invalid inputs

	     printf("Entered R/Rsun = %f\n\n", rratio);
//
// Compute the average and the central density for n=3 polytrope.
//
	     printf("\n\n\n\n");
	     printf("POLYTROPIC STAR SKETCH RESULTS - Eddington Standard Model, n=3.\n\n");
	     rstar = rratio*rsun;
	     mstar = mratio*msun;
	     if( xhydro != 0.0 && yhelium != 0.0 && zheavy != 0.0)
	        printf("Chemistry: X = %f,  Y = %f,  Z = %f .\n", xhydro, yhelium, zheavy );
	     printf("Molecular weight = mu = %f.  Polytropic gamma = 4/3.\n", mu);
	     printf("R/Rsun = %f,  R = %e meters.\n",rratio,rstar);
	     printf("M/Msun = %f,  Mass = %e kg.\n\n", mratio, mstar);       

	     
	     avgdensity = mstar/((4/3)*3.14159*(rstar*rstar*rstar));
	     densitycentral = avgdensity*54.1825;
//
// Compute the central total pressure for n=3 polytrope
//             = (GM**2/R**4)*[(n+1)4pi(dtheta/dx)**2)]**-1
//
	     pressurecentral =   mstar*mstar*6.67e-11/(rstar*rstar*rstar*rstar);
	     factor = 4.0*3.14159*4.0*(0.04231)*(0.04231);
	     factor = 1.0/factor;
	     pressurecentral = pressurecentral*factor;

	     printf("Averge density = %e kg/m**3.\n",avgdensity);
	     printf("Central density= %e kg/m**3.\n", densitycentral);
	     printf("Central total pressure = %e kg/m**2.\n", pressurecentral);
//
// find (1 - beta)/beta**4 = ( (Pc**3)/ro_c**4 )*[(mu*mh/k)**4]*(a/3).
//
	     fbeta = pressurecentral*pressurecentral*pressurecentral;
	     fbeta = fbeta/(densitycentral*densitycentral*densitycentral*densitycentral);
	     fbeta = fbeta*(mu*mh/k)*(mu*mh/k)*(mu*mh/k)*(mu*mh/k);
	     fbeta = (a/3.0)*fbeta;

	     printf("[1 - beta]/beta**4 = %e.\n", fbeta);

	     beta = findbeta(fbeta);
	     printf("Beta = %e. Pgas = beta*Ptotal, Pradiation = (1-beta)*Ptotal.\n", beta);
	     pgas = beta*pressurecentral;
	     pradiation = (1.0 - beta)*pressurecentral;
	     pratio = pradiation/pgas;

	     printf("Pradiation/Ptotal = %e. \n", 1.0 - beta);
	     printf("Pgas/Ptotal = %e.\n", beta);
	     printf("Pradiation/Pgas = %e.\n", pratio );

	     tc = (mu*mh/(k*densitycentral))*beta*pressurecentral;
	     printf("Tcentral = %e degrees Kelvin.\n",tc);

	     omega = -(3.0/(5.0 - 3.0))*(6.67e-11)*mstar*mstar/(rstar);
             printf("Stellar gravitational binding energy =  %e Joules.\n",omega);

	     printf("\nEnter 0 to restart STAR SKETCH, or");
	     printf(" 1 to do another polytrope: ");
	     scanf("%s",value);   // This
       	     d = atof(value);     // entry procedure is more immune to invalid inputs
       	     pselect = d;      // than simple scanf().

             printf("\n");
	     if(pselect == 0)
			return;
	  }
	  if(chemsw == 5)
	      pselect = 0;    // drop through loop and return.
	  if(chemsw == 6 || chemsw == 0 ){
	       sublist();
	       printf("Enter 1 to continue: ");
	       scanf("%s",value);   // This
       	       pauseflag = atof(value);     // entry procedure is more immune to invalid inputs
          }
	  if(chemsw == 7){
	       printf("EXIT CHANDRA");
	       exit(0);
	  } 
	}	
}
/************************************************************
*
*  findbeta - solve (1 - b)/b**4 = C < 1 for ( 0 <= b <= 1 ).
*             Note that since this function wiggles in the
*             interval and is not monotone increasing or
*             decreasing, we have to scan the interval
*             for the best fit.
*
************************************************************/
double findbeta(double fbetanumber)
{
     int cnt;				// counter.
     double fbeta;  // (1-b)/b**4   = fbetanumber when b is known.     
     double b;   // beta.
     double bbest; // best beta in scan of interval
     double epsilon;
     double deltabest;
     double delta;

     epsilon = 1.0/30000.0;  // use 30,000 steps.     
     cnt = 0;
     b = 0.0;
     deltabest = 1.0e8;
     while(cnt < 30000){
        b = b + epsilon;
	fbeta = (1.0 - b)/(b*b*b*b);
	delta = sqrt((fbetanumber - fbeta)*(fbetanumber - fbeta));
	if(delta < deltabest){
	    deltabest = delta;
	    bbest = b;
        }
        cnt++;
     }
     return(bbest);
}
/**************************************************************
*
*  distancecal() - compute the distance from the parallax.
*
***************************************************************/
void distancecal(void)
{
     double parallax; // angle in sec,  r = D_earth-sun/angle-in-radians.
     double pangledeg; // angle in degrees.
     double pangle;   // angle in radians. 
     double r;        // distance in light-years.
     double rpc;      // distance in parsecs.
     double dearthsun = 0.1581395e-4; // distance earth-sun in LY.
     int sustain;

     sustain = 1;
     while(sustain == 1){
	printf("\nCOMPUTE DISTANCE FROM PARALLAX.\n\n");
        printf(" Parallax to typical stars:\n");
        printf(" Sirius         -> 0.377 seconds of arc.\n");
        printf(" Alpha Centauri -> 0.753 seconds of arc.\n");
        printf("\nEnter the floating-point parallax in seconds of arc: ");
	scanf("%s",value);   // This
        parallax = atof(value);     // entry procedure is more immune to invalid inputs

        pangledeg = parallax/(60.0*60.0); // compute angle in degrees
        pangle = pangledeg*(3.14159/180.0);    // comp angle in radians
        r = dearthsun/pangle;  // comp distance in ly.
        rpc = r/3.2600  ;  // comp distance in parsecs. 
        printf("\n");
        printf("parallax = %e seconds, r = %e light-years.\n", parallax, r);
        printf("         = %e degrees, r = %e parsecs.\n", pangledeg, rpc);
	printf("         = %e radians. r = %e meters.\n", pangle, r*9.46e15 );
	printf("\n");
	printf("ENTER 0 to return, or 1 to continue computing parallaxes: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        sustain = d;      // than simple scanf().

	printf("Entered %d .\n\n", sustain);
     }
     
     return;
}
/**************************************************************
*
*  luminosity() - Compute L/Lsun, M/Msun, R/Rsun & M
*                 from r in LY, m, and Tsurface,
*                 including the bolometric correction BC.
*
*
***************************************************************/
void luminosity(void)
{
     int sustain;
     double r;		// distance to the star in LY.
     double rpc;        // distance to the star in parsecs.
     double m;          // visual magnitude m.
     double absmag;     // absolute magnitude M.
     double lratio;     // L/Lsun luminosity.
     double expo;       // exponent, L/Lsun = 10**exponent.
     double mratio;     // M/Msun mass ratio.
     double mratiolow;  // M/Msun mass ratio.
     double mratiobest; // M/Msun for x = 3.409 best fit to mass-luminosity data.
     double rratio;     // R/Rsun.
     double x = 3.0;    // L/Lsun = (M/Msun)**x.   3 <= x < 4.
     double tsurface;   // suface temperature of star for BC, bolometric correction.
     double bc;		// M.Schwarzschild bolometric correction, pg. 9.
     double xt;		// xt = log10(Tsurface).
//
// need to insert the bolometric correction, a function of the spectrum.
//

     sustain = 1;
     while(sustain == 1){
        x = 3.0;
        printf("\nCOMPUTE L/Lsun, M/Msun,& absolute magnitude M\n");
	printf("from r in light-years and the visual magnitude m, including\n");
	printf("the bolometric correcton BC for atmospheric absorption.\n\n");

	printf("Enter the floating-point distance to the star in light-years: ");
	scanf("%s",value);   // This
        r = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered %f light-years.\n",r);
	printf("\nEnter the floating-point visual magnitude m on the charts: ");
	scanf("%s",value);   // This
        m = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered m = %f\n",m);

	printf("\nEnter Tsurface in deg. Kelvin for bolometric correction: ");
	scanf("%s",value);   // This
        tsurface = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered %e Kelvin.\n", tsurface);

	xt = log10(tsurface);  // find the look-up value for BCstellar.

	bc = bolometric(xt);  // from the look-up routine for bolometric corrections.

        rpc = r/3.2600;  // compute distance in parsecs.
	absmag = m + 5.0 - (5.0*log10(rpc));   // cal absolute magnitude M.
        absmag = absmag + bc;                  // include stellar bolometric BC.
        expo = (4.761176 - absmag)/2.5;       
	lratio = exp(2.302585*expo); // e**Ax = 10**x, A = 2.302585
	printf("\nL/Lsun = %e.  Bolometric correction BC = %f\n",lratio,bc);
//
// L/Lsun = (M/Msun)**x, where x = about 3, so M/Msun = (L/Lsun)**(1/x).
// Then log10(M/Msun) = (1/x)log10[L/Lsun],
// so M/Msun = 10**[(1/x)*log10[L/Lsun] = exp[(2.302585/x)*log10[L/Lsun].
//
	mratio = exp((2.302585/x)*log10(lratio));
	x = 4.0;
	mratiolow =  exp((2.302585/x)*log10(lratio));
	printf("%f < M/Msun < %f,  3 < x < 4.\n", mratiolow, mratio);

	x = 3.409; // empirical best guess from chart measurements.
	mratiobest =  exp((2.302585/x)*log10(lratio));
	printf("M/Msun = %f , x = %f  best mass-luminosity chart fit.\n", mratiobest, x);

// Mass-Luminosity Chart:
//   See THERMONUCLEAR FUSION IN STARS, by James A.Green, Figure 1.25, page 34.

	rratio = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lratio);
        printf("R/Rsun = %e\n",rratio);

	printf("Visual magnitude   m = %f\n", m);
	printf("Absolute magnitude with bolometric correction M + B.C. = %f\n", absmag);
	printf("r = %e light-years = %e parsecs.\n",r, rpc);
	printf("\nENTER 1 to continue calc from r, m, & Tsurface or 0 to return: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        sustain = d;      // than simple scanf().

        printf("Entered %d\n",sustain);
     }
     return;
}
/************************************************************
*
*    bolometric(xt) - Return bolometric correction from
*                     xt = log10(Tsurface).
*
************************************************************/
double bolometric(double xt)
{
       double bc;
// 
// bcflag == 0->Use Zombeck [9] 1990 values. == 1->Use Schwarzschild [8] values.
//
// bcflag is a compile-time option of STARVIEW & CHANDRA.
//
// need to insert the bolometric correction, a function of the spectrum.
//
//  L/Lsun = 10**{[4.761176 - [M_v + B.C.]]/2.5}
//
//  (L/Lsun)(r_sun/r)**2 = I/Isun = 10**(2/5)(m_sun_bol - m_bol).
//
//  m_sun_bol = 4.761176, m_bol = m_v + B.C.
//                        M_bol = M_v + B.C.
//
//  L/Lsun = 10**(2/5)(M_sun_bol - M_bol) =  10**[(2/5)(M_sun_bol - M_v)][10**(-2/5)B.C.]
//
//         = [(r/r_sun)**2]*{10**[(2/5)(m_sun_bol - m_v)]}[10**(-2/5)B.C.],
//
//   where r/r_sun is the distance to the star in AU.
//         
//   M.Schwarzschild B.C. (1950s)          Martin V. Zombeck (1990) values
//   STRUCTURE & EVOLUTION OF THE          HANDBOOK OF SPACE ASTRONOMY &
//   STARS, pg.9.                          ASTROPHYSICS [9], derived from p.68-69.
//   (after Balloon measurements?)          (after satellite measurements?) Origin:
//                                    Lang, K.,Astrophysical Formulae,Springer Verlag,1974.
//
//  log10(Tsurface)    BCstar              log10(Tsurface)   BCstar
//	4.65		-4.1               4.5               -3.34
//	4.46		-3.0               4.48              -3.17
//	4.32		-2.3               4.43              -2.8
//	4.21		-1.7               4.38              -2.5
//	4.13		-1.25              4.34              -2.23
//	4.08		-0.95              4.27              -1.77
//	4.03		-0.72              4.21              -1.39
//	3.99		-0.52              4.18              -1.21
//	3.95		-0.38              4.16              -1.04
//	3.91		-0.2               4.13              -0.85
//	3.87		-0.09              4.09              -0.66
//	3.83		-0.02              4.03              -0.4
//	3.8		-0.0               3.98              -0.25
//	3.76		-0.03              3.935             -0.15
//	3.73		-0.09              3.913             -0.12
//	3.69		-0.18              3.86              -0.08
//	3.66	        -0.29              3.84              -0.06
//      3.63		-0.42              3.815             -0.04
//	3.61		-0.57              3.79              -0.05
//	3.58		-0.74              3.77              -0.06
//	3.56		-0.92              3.76              -0.07
//	3.55		-1.11              3.74              -0.10
//     	3.53		-1.3               3.73              -0.15
//	3.52		-1.5               3.72              -0.19
//	3.52		-1.7               3.68              -0.25
//	3.51		-1.9               3.64              -0.65
//                                         3.62              -0.9
//                                         3.59              -1.2
//                                         3.55              -1.48
//                                         3.54              -1.76
//                                         3.53              -2.03
//                                         3.51              -2.31
//                                         3.49              -2.62
//                                         3.42              -4.2
//   
//
    if(bcflag == 1){ //Choose between Schwarzschild [8] & Zombeck [9] scheme corrections.
	if(xt > 4.65)     //  1->Schwarzschild [8] B.C. corrections, 1950s.
	   bc = -4.1;     //  install estimates for ends of range.
	if(xt < 3.51)     // bcflag is a compile-time option.
	   bc = -1.9;
	if(xt <= 4.65 && xt >= 4.46){
	   bc = (4.65 - xt)*((4.1 - 3.0)/(4.65 - 4.46)) - 4.1;
	   bcprobe(bc,1);
	}
	if(xt <= 4.46 && xt >= 4.32){
	   bc = (4.46 - xt)*((3.0 - 2.3)/(4.46 - 4.32)) - 3.0;
	   bcprobe(bc,2);
	}
	if(xt <= 4.32 && xt >= 4.21){
	   bc = (4.32 - xt)*((2.3 - 1.7)/(4.32 - 4.21)) - 2.3;
	   bcprobe(bc,3);
	}
	if(xt <= 4.21 && xt >= 4.13){
	   bc = (4.21 - xt)*((1.7 - 1.25)/(4.21 - 4.13)) - 1.7;
	   bcprobe(bc,4);
	}
	if(xt <= 4.13 && xt >= 4.08){
	   bc = (4.13 - xt)*((1.25 - 0.95)/(4.13 - 4.08)) - 1.25;
	   bcprobe(bc,5);
	}
	if(xt <= 4.08 && xt >= 4.03){
	   bc = (4.08 - xt)*((0.95 - 0.72)/(4.08 - 4.03)) - 0.95;
	   bcprobe(bc,6);
	}
	if(xt <= 4.03 && xt >= 3.99){
	   bc = (4.03 - xt)*((0.72 - 0.52)/(4.03 - 3.99)) - 0.72;
	   bcprobe(bc,7);
	}
	if(xt <= 3.99 && xt >= 3.95){
	   bc = (3.99 - xt)*((0.52 - 0.38)/(3.99 - 3.95)) - 0.52;
	   bcprobe(bc,1);
	}
	if(xt <= 3.95 && xt >= 3.91){
	   bc = (3.95 - xt)*((0.38 - 0.20)/(3.95 - 3.91)) - 0.38;
	   bcprobe(bc,8);
	}
	if(xt <= 3.91 && xt >= 3.87){
	   bc = (3.91 - xt)*((0.20 - 0.09)/(3.91 - 3.87)) - 0.20;
	   bcprobe(bc,9);
	}
	if(xt <= 3.87 && xt >= 3.83){
	   bc = (3.87 - xt)*((0.09 - 0.02)/(3.87 - 3.83)) - 0.09;
	   bcprobe(bc,10);
	}
	if(xt <= 3.83 && xt >= 3.8){
	   bc = (3.83 - xt)*((0.02 - 0.00)/(3.83 - 3.80)) - 0.02;
	   bcprobe(bc,11);
	}
	if(xt <= 3.8 && xt >= 3.76){
	   bc = (3.8 - xt)*((0.0 - 0.03)/(3.80 - 3.76)) - 0.0;
	   bcprobe(bc,12);
	}
	if(xt <= 3.76 && xt >= 3.73){
	   bc = (3.76 - xt)*((0.03 - 0.09)/(3.76 - 3.73)) - 0.03;
	   bcprobe(bc,13);
	}
	if(xt <= 3.73 && xt >= 3.69){
	   bc = (3.73 - xt)*((0.09 - 0.18)/(3.73 - 3.69)) - 0.09;
	   bcprobe(bc,14);
	}
	if(xt <= 3.69 && xt >= 3.66){
	   bc = (3.69 - xt)*((0.18 - 0.29)/(3.69 - 3.66)) - 0.18;
	   bcprobe(bc,15);
	}
	if(xt <= 3.66 && xt >= 3.63){
	   bc = (3.66 - xt)*((0.29 - 0.42)/(3.66 - 3.63)) - 0.29;
	   bcprobe(bc,16);
	}
	if(xt <= 3.63 && xt >= 3.61){
	   bc = (3.63 - xt)*((0.42 - 0.57)/(3.63 - 3.61)) - 0.42;
	   bcprobe(bc,17);
	}
	if(xt <= 3.61 && xt >= 3.58){
	   bc = (3.61 - xt)*((0.57 - 0.74)/(3.61 - 3.58)) - 0.57;
	   bcprobe(bc,18);
	}
	if(xt <= 3.58 && xt >= 3.56){
	   bc = (3.58 - xt)*((0.74 - 0.92)/(3.58 - 3.56)) - 0.74;
	   bcprobe(bc,19);
	}
	if(xt <= 3.56 && xt >= 3.55){
	   bc = (3.56 - xt)*((0.92 - 1.11)/(3.56 - 3.55)) - 0.92;
	   bcprobe(bc,20);
	}
	if(xt <= 3.55 && xt >= 3.53){
	   bc = (3.55 - xt)*((1.11 - 1.3)/(3.55 - 3.53)) - 1.11;
	   bcprobe(bc,21);
	}
	if(xt <= 3.53 && xt >= 3.52){
	   bc = (3.53 - xt)*((1.3 - 1.5)/(3.53 - 3.52)) - 1.3;
	   bcprobe(bc,22);
	}
	if(xt <= 3.52 && xt >= 3.51){
	   bc = (3.52 - xt)*((1.5 - 1.9)/(3.52 - 3.51)) - 1.5;
	   bcprobe(bc,23);
	}
	return(bc);
    }
    else if(bcflag == 0){  // Conditionally use 0->Zombeck[9] table values, p68-69.
	if(xt > 4.5)     //  Martin Zombeck [9] B.C. corrections, circa 1990.
	   bc = -3.34;     //  install estimates for ends of range.
	if(xt < 3.42)      // bcflag is a compile-time option.
	   bc = -4.2;
	if(xt <= 4.5 && xt >= 4.48){
	   bc = (4.5 - xt)*((3.34 - 3.17)/(4.5 - 4.48)) - 3.34;
           bcprobe(bc,1);
	}
	if(xt <= 4.48 && xt >= 4.43){
	   bc = (4.48 - xt)*((3.17 - 2.8)/(4.48 - 4.43)) - 3.17;
	   bcprobe(bc,2);
	}
	if(xt <= 4.43 && xt >= 4.38){
	   bc = (4.43 - xt)*((2.8 - 2.5)/(4.43 - 4.38)) - 2.8;
	   bcprobe(bc,3);
	}
	if(xt <= 4.38 && xt >= 4.34){
	   bc = (4.38 - xt)*((2.5 - 2.23)/(4.38 - 4.34)) - 2.5;
	   bcprobe(bc,4);
	}
	if(xt <= 4.34 && xt >= 4.27){
	   bc = (4.34 - xt)*((2.23 - 1.77)/(4.34 - 4.27)) - 2.23;
	   bcprobe(bc,5);
	}
	if(xt <= 4.27 && xt >= 4.21){
	   bc = (4.27 - xt)*((1.77 - 1.39)/(4.27 - 4.21)) - 1.77;
           bcprobe(bc,6);
	}
	if(xt <= 4.21 && xt >= 4.18){
	   bc = (4.21 - xt)*((1.39 - 1.21)/(4.21 - 4.18)) - 1.39;
           bcprobe(bc,7);
	}
	if(xt <= 4.18 && xt >= 4.16){
	   bc = (4.18 - xt)*((1.21 - 1.04)/(4.18 - 4.16)) - 1.21;
           bcprobe(bc,8);
	}
	if(xt <= 4.16 && xt >= 4.13){
	   bc = (4.16 - xt)*((1.04 - 0.85)/(4.16 - 4.13)) - 1.04;
           bcprobe(bc,9);
	}
	if(xt <= 4.13 && xt >= 4.09){
	   bc = (4.13 - xt)*((0.85 - 0.66)/(4.13 - 4.09)) - 0.85;
           bcprobe(bc,10);
	}
	if(xt <= 4.09 && xt >= 4.03){
	   bc = (4.09 - xt)*((0.66 - 0.4)/(4.09 - 4.03)) - 0.66;
           bcprobe(bc,11);
	}
	if(xt <= 4.03 && xt >= 3.98){
	   bc = (4.03 - xt)*((0.4 - 0.25)/(4.03 - 3.98)) - 0.4;
           bcprobe(bc,12);
	}
	if(xt <= 3.98 && xt >= 3.935){
	   bc = (3.98 - xt)*((0.25 - 0.15)/(3.98 - 3.935)) - 0.25;
           bcprobe(bc,13);
	}
	if(xt <= 3.935 && xt >= 3.913){
	   bc = (3.935 - xt)*((0.15 - 0.12)/(3.935 - 3.913)) - 0.15;
           bcprobe(bc,14);
	}
	if(xt <= 3.913 && xt >= 3.86){
	   bc = (3.913 - xt)*((0.12 - 0.08)/(3.913 - 3.86)) - 0.12;
           bcprobe(bc,15);
	}
	if(xt <= 3.86 && xt >= 3.84){
	   bc = (3.86 - xt)*((0.08 - 0.06)/(3.86 - 3.84)) - 0.08;
           bcprobe(bc,16);
	}
	if(xt <= 3.84 && xt >= 3.815){
	   bc = (3.84 - xt)*((0.06 - 0.04)/(3.84 - 3.815)) - 0.06;
           bcprobe(bc,17);
	}
	if(xt <= 3.815 && xt >= 3.79){
	   bc = (3.815 - xt)*((0.04 - 0.05)/(3.815 - 3.79)) - 0.04;
           bcprobe(bc,18);
	}
	if(xt <= 3.79 && xt >= 3.77){
	   bc = (3.79 - xt)*((0.05 - 0.06)/(3.79 - 3.77)) - 0.05;
           bcprobe(bc,19);
	}
	if(xt <= 3.77 && xt >= 3.76){
	   bc = (3.77 - xt)*((0.06 - 0.07)/(3.77 - 3.76)) - 0.06;
           bcprobe(bc,20);
	}
	if(xt <= 3.76 && xt >= 3.74){
	   bc = (3.76 - xt)*((0.07 - 0.10)/(3.76 - 3.74)) - 0.07;
           bcprobe(bc,21);
	}
	if(xt <= 3.74 && xt >= 3.73){
	   bc = (3.74 - xt)*((0.10 - 0.15)/(3.74 - 3.73)) - 0.10;
           bcprobe(bc,22);
	}
	if(xt <= 3.73 && xt >= 3.72){
	   bc = (3.73 - xt)*((0.15 - 0.19)/(3.73 - 3.72)) - 0.15;
           bcprobe(bc,23);
	}
	if(xt <= 3.72 && xt >= 3.68){
	   bc = (3.72 - xt)*((0.19 - 0.25)/(3.72 - 3.68)) - 0.19;
           bcprobe(bc,24);
	}
	if(xt <= 3.68 && xt >= 3.64){
	   bc = (3.68 - xt)*((0.25 - 0.65)/(3.68 - 3.64)) - 0.25;
           bcprobe(bc,25);
	}
	if(xt <= 3.64 && xt >= 3.62){
	   bc = (3.64 - xt)*((0.65 - 0.9)/(3.64 - 3.62)) - 0.65;
           bcprobe(bc,26);
	}
	if(xt <= 3.62 && xt >= 3.59){
	   bc = (3.62 - xt)*((0.9 - 1.2)/(3.62 - 3.59)) - 0.9;
           bcprobe(bc,27);
	}
	if(xt <= 3.59 && xt >= 3.55){
	   bc = (3.59 - xt)*((1.2 - 1.48)/(3.59 - 3.55)) - 1.2;
           bcprobe(bc,28);
	}
	if(xt <= 3.55 && xt >= 3.54){
	   bc = (3.55 - xt)*((1.48 - 1.76)/(3.55 - 3.54)) - 1.48;
           bcprobe(bc,29);
	}
	if(xt <= 3.54 && xt >= 3.53){
	   bc = (3.54 - xt)*((1.76 - 2.03)/(3.54 - 3.53)) - 1.76;
           bcprobe(bc,30);
	}
	if(xt <= 3.53 && xt >= 3.51){
	   bc = (3.53 - xt)*((2.03 - 2.31)/(3.53 - 3.51)) - 2.03;
           bcprobe(bc,31);
	}
	if(xt <= 3.51 && xt >= 3.49){
	   bc = (3.51 - xt)*((2.31 - 2.62)/(3.51 - 3.49)) - 2.31;
           bcprobe(bc,32);
	}
	if(xt <= 3.49 && xt >= 3.42){
	   bc = (3.49 - xt)*((2.62 - 4.2)/(3.49 - 3.42)) - 2.62;
	   bcprobe(bc,33);
	}	
	return(bc);
    }
}
void bcprobe(double bc, int i) // test for non-negative B.C. and stop with position display.
{
	if(bc > 0.0){
	    printf("Error at probe[%d], bc > 0.0.",i);
            printf("Enter 1 to proceed :");
	    scanf("%s",value);       // Entry procedure more immune to
	    d = atof(value);         // invalid inputs than simple scanf
	} 
}

/**************************************************************
*
*  luminosity2() - Cal L/Lsun, M/Msun, R/Rsun & M
*                  from r in LY and m,
*                  without the bolometric correction BC.
*
*
***************************************************************/
void luminosity2(void)
{
     int sustain;
     double r;		// distance to the star in LY.
     double rpc;        // distance to the star in parsecs.
     double m;          // visual magnitude m.
     double absmag;     // absolute magnitude M.
     double lratio;     // L/Lsun luminosity.
     double expo;       // exponent, L/Lsun = 10**exponent.
     double mratio;     // M/Msun mass ratio. For x=3.
     double mratiolow;  // M/Msun mass ratio. For x=4.
     double mratiobest; // M/Msun mass ratio. For x=3.409 best chart fit.
     double rratio;     // R/Rsun.
     double x = 3.0;    // L/Lsun = (M/Msun)**x.   3 <= x < 4.
     double tsurface;   // surface temperature.
//
// Need to insert bolometric correction, a function of the spectrum.
//
     sustain = 1;
     while(sustain == 1){
        x = 3.0;
        printf("\nCOMPUTE L/Lsun, M/Msun,& absolute magnitude M\n");
	printf("from r in parsecs and the visual magnitude m, WITHOUT\n");
	printf("the bolometric correction for atmospheric absorption.\n\n");

	printf("Enter the floating-point distance to the star in light-years: ");
	scanf("%s",value);   // This
        r = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered %f light-years.\n",r);
	printf("\nEnter the floating-point visual magnitude m on the charts: ");
	scanf("%s",value);   // This
        m = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered m = %f\n",m);
	printf("\nEnter floating-point Tsurface in degrees Kelvin : ");
	scanf("%s",value);   // This
        tsurface = atof(value);     // entry procedure is more immune to invalid inputs

	printf("Entered %e Kelvin.\n\n", tsurface);

        rpc = r/(3.2600);  // compute distance in parsecs.
	absmag = m + 5.0 - (5.0*log10(rpc));   // cal absolute magnitude M.
        expo = (4.761176 - absmag)/2.5;       
	lratio = exp(2.302585*expo); // e**Ax = 10**x, A = 2.302585
	printf("\nL/Lsun = %e.\n",lratio);
//
// L/Lsun = (M/Msun)**x, where x = about 3, so M/Msun = (L/Lsun)**(1/x).
// Then log10(M/Msun) = (1/x)log10[L/Lsun],
// so M/Msun = 10**[(1/x)*log10[L/Lsun] = exp[(2.302585/x)*log10[L/Lsun].
//
	mratio = exp((2.302585/x)*log10(lratio));
        x = 4.0;
	mratiolow =  exp((2.302585/x)*log10(lratio));
	printf("%f < M/Msun < %f,   3 < x < 4.\n", mratiolow, mratio);

	x = 3.409;
	mratiobest =  exp((2.302585/x)*log10(lratio));
	printf("M/Msun = %f , x = %f , best mass-luminosity chart fit.\n", mratiobest,x);

// Mass-Luminosity Chart:
//   See THERMONUCLEAR FUSION IN STARS, by James A.Green, Figure 1.25. page 34.

        rratio = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lratio);
	printf("R/Rsun = %e\n",rratio);

	printf("Visual magnitude   m = %f, absolute magnitude M = %f\n", m, absmag);
	printf("r = %e light-years = %e parsecs.\n",r, rpc);
	printf("\nENTER 1 to continue calc from r, m, and Tsurface or 0 to return: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        sustain = d;      // than simple scanf().

        printf("Entered %d\n",sustain);
     }
     return;
}
/**************************************************************
*
*  stellarmass() - compute M/Msun from L/Lsun.
*
***************************************************************/
void stellarmass(void)
{
     int sustain;
     double lratio;     // L/Lsun luminosity.
     double expo;       // exponent, L/Lsun = 10**exponent.
     double mratio;     // M/Msun mass ratio.
     double x = 3.0;    // L/Lsun = (M/Msun)**x.   3 <= x < 4.

     x = 3.0;   // set the mass-luminosity law exponent.
     sustain = 1;
     while(sustain == 1){
	printf("\nCOMPUTE M/Msun from L/Lsun.\n\n");
	printf("Enter floating-point L/Lsun : ");
	scanf("%s",value);   // This
        lratio = atof(value);     // entry procedure is more immune to invalid inputs

//
// L/Lsun = (M/Msun)**x, where x = about 3, so M/Msun = (L/Lsun)**(1/x).
// Then log10(M/Msun) = (1/x)log10[L/Lsun],
// so M/Msun = 10**[(1/x)*log10[L/Lsun] = exp[(2.302585/x)*log10[L/Lsun].
//
	mratio = exp((2.302585/x)*log10(lratio));
	printf("M/Msun = %f,  L/Lsun = %e\n", mratio, lratio);
	
	printf("\nENTER 1 to continue M/Msun calc or 0 to return: ");
	scanf("%s",value);   // This
        d = atof(value);     // entry procedure is more immune to invalid inputs
        sustain = d;      // than simple scanf().

        printf("Entered %d\n",sustain);
     }
     return;
}
/**************************************************************
*
*  stellarradius() - compute R/Rsun from L/Lsun & Tsurface.
*
***************************************************************/
void stellarradius(void)
{
     int sustain;
     double lratio;  // L/Lsun
     double tsurface;   // Tsurface of star in degrees Kelvin.
     double rratio;  // R/Rsun
//
//   L = [4*pi*R**2]*sigma*T**4, so L/Lsun = [[R/Rsun]**2]*[T/Tsun]**4,
//             so R/Rsun = [(Tsun/T)**2]*[[L/Lsun]**(1/2)].

     sustain = 1;
     while(sustain == 1){   
	  printf("\nCOMPUTE R/Rsun stellar radius from Tsurface and L/Lsun.\n\n");
	  printf("Enter floating-point Tsurface in degrees Kelvin : ");
	  scanf("%s",value);   // This
          tsurface = atof(value);     // entry procedure is more immune to invalid inputs

	  printf("Entered %e Kelvin.\n\n", tsurface);
	  printf("Enter L/Lsun : ");
	  scanf("%s",value);   // This
          lratio = atof(value);     // entry procedure is more immune to invalid inputs

	  printf("Entered L/Lsun = %e\n", lratio);
	  rratio = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lratio);
	  printf("\nR/Rsun = %e\n",rratio);
	  printf("\nEnter 1 to continue R/Rsun calc, or 0 to return: ");  

	  scanf("%s",value);   // This
          d = atof(value);     // entry procedure is more immune to invalid inputs
          sustain = d;      // than simple scanf().

	  printf("Entered %d\n", sustain);
     }
     return;
}
/**************************************************************
*
*  tsurf() - compute Tsurface from the spectral type.
*
***************************************************************/
void tsurf(void)
{
     int proceedflag = 0;

     printf("Spectral Type and Tsurface in degrees Kelvin (after Zombeck)\n");
     printf("   Regular Main Sequence Stars [V]    || Giant Stars [III]\n");
     printf("Type  Temp    Type Temp   Type  Temp  || Type  Temp\n");
     printf("O5-O7 38000K  A0  10800K  K0    5240K || G5    5010K\n");
     printf("O8-O9 35000K  A2   9730K  K2    4780K || G8    4870K\n");
     printf("O9.5  31900K  A5   8620K  K5    4558K?|| K0    4720K\n");
     printf("              A7   8190K  K7    4410K || K2    4460K\n");
     printf("B0    30000K                          || K3    4210K\n");                    
     printf("B0.5  27000K  F0   7240K  M0    3920K || K4    4010K\n");
     printf("B1    24200K  F2   6930K  M1    3680K || K5    3780K\n");
     printf("B2    22100K  F5   6540K  M2    3500K || M0    3660K\n");
     printf("B3    18800K  F8   6200K  M3    3360K || M1    3600K\n");
     printf("B5    16400K              M4    3230K || M2    3500K\n");
     printf("B6    15400K  G0   5920K  M5    3120K || M3    3300K\n");
     printf("B7    14500K  G2   5780K  M6    2500K?|| M4    3100K\n");
     printf("B8    13400K  G5   5610K  M7    2380K?|| M5    2950K\n");
     printf("B9    13400K  G8   5490K  M8    2260K || M6    2800K\n");
     printf("Spectral Type Characteristics\n");
     printf("O > 25000K,          B 11000K < T < 25000K, A 7500K < T < 11000K\n");
     printf("F 6000K < T < 7500K, G 5000K < T < 6000K,   K 3500K < T < 5000K\n");
     printf("M 2200K < T < 3500K. Classes with different spectral lines are:\n");
     printf("S 2200K < T < 3500K, R(C) 5000K < T < 3500K,N(C) 3500 < T < 2200,\n");
     printf("W > 25000K,          WC > 25000K,           WN > 25000K [Wolf-Rayets]\n");
     printf("ENTER 1 to continue: ");
     scanf("%s",value);   // This
     d = atof(value);     // entry procedure is more immune to invalid inputs
     proceedflag = d;      // than simple scanf().


     return;
}
/**************************************************************
*
*   listmainseq() - List properties of well-known Main Sequence
*                   stars for use in polytrope() sketch routine
*                   above.
* 		  
*
****************************************************************/
void listmainseq(void)
{
	int pselect = 1;
        void sublist(void);

        while(pselect==1){
	  sublist(); // list typical main-sequence stars.
          printf("Enter 0 to exit CHANDRA, or \n");
	  printf("      1 to do a basic stellar variable or polytropic sketch:");
	  scanf("%s",value);   // This
          d = atof(value);     // entry procedure is more immune to invalid inputs
	  pselect = d;      // than simple scanf().

	  if(pselect == 0){
		printf("EXIT CHANDRA");
                exit(0);
	  }
	  if(pselect == 1)
             polytrope();
	}
	return;
}
void sublist(void)
{
        printf("Properties of well-known Main Sequence Stars & Polytrope Sketch.\n");
	printf("M.Schwarzschild Main Sequence chemistry X=0.75, Y=0.23, Z=0.02\n");
	printf("STAR       Derived Properties [Book] || Observed Properties || III=Giant\n");
	printf("NAME(Constella) M/Msun   R/Rsun  L/Lsun Distance mag m Spectrum->Tsurf\n");
	printf("Sun             1.0      1.0     1.0    1 AU    -26.74  G2       5770K\n");
	printf("Aldebaran(Tau)          40.0(bhm)125    68 LY    0.86   K5 III   3400K\n");
	printf("Altair(Aquila)                   11     16 LY    0.77   A7 V     8190K\n");
	printf("Antares(Sco)   10-15     700(bhm)9000   520 LY   0.92   M1 Ib    3100K\n");
	printf("Arcturus(Boo)    4        25(bhm)115    37 LY   -0.06   K2 III   4200K\n");
	printf("Bellatrix(Ori)                   4000   470 LY   1.64   B2 III   22100K\n");
	printf("Deneb(Cygnus)   25        60(bhm)60,000 1600 LY  1.26   A2 Ia    9700K\n");
	printf("Denebola(Leo)                    20     43 LY    2.14   A3 V     9360K\n");
	printf("Duhbe(Ursa Ma)                   145    105 LY   1.81   K0 III   5249K\n");
	printf("Eltanin(Draco)                   145    100 LY   2.22   K5 III   3780K\n");
	printf("Enif(Pegasus)                    5800   780 LY   2.31   K2 Ib    4460K\n");
	printf("Fomalhaut(P.A)             2(bhm)14     23 LY    1.17   A3 V     8990K\n");
	printf("Gienah(Corvus)                   1200   450 LY   2.6    B8 III   13400K(zom)\n");
	printf("Geinah(Cygnus)                   40      75 LY   2.46   K0 III   4720K\n");
	printf("Agena(Centaur)                   10,000 490 LY   0.66   B1 II    24200K?B1 V\n");
	printf("Hamal(Aries)                     70      75 LY   2.00   K2 III   4210K\n");
	printf("Regulus(Leo)              5(bhm) 160     85 LY   1.36   B7 V     13000K\n");
	printf("Rigel(Orion)     50      50(bhm) 57,000 900 LY   0.14   B8 Ia    12000K\n");
	printf("Sirius(Ca.Ma.)   2.35   1.8(bhm) 23     8.7 LY  -1.42   A1       10265K\n");
        return;
}

/*
*     References
*
* [1] - Richard Bowers & Terry Deeming, 1984. Astrophysics I: The Stars.
*       Jones & Barlett.
* [2] - Stuart L. Shapiro & Saul A. Teukolsky, 1983.
*       Black Holes, White Dwarfs, and Neutron Stars. Wiley-Interscience.
* [3] - W. David Arnett and Richard L. Bowers, 1977.
*       "A Microscopic Interpretation of Neutron Star Structure",
*       Supplement to the Astrophysical Journal, V.33, p. 415.
* [4] - Lee Lindblom & Steven L. Detweiler, 1983.
*       "The Quadropole Oscillations of Neutron Stars",
*       Supplement to the Astrophysical Journal, V.53, p.73.
* [5] - Green, James A., CHANDRA: Chandrasekhar Sets of White Dwarfs
*           and Supernova Cores with Stellar Structure Software, 1997.
* [6] - Green, James A., THERMONUCLEAR FUSION IN STARS, Greenwood
*          Research Books & Software, 1999.
*
*       The author thanks S. Chandrasekhar of the Fermi Institute at
*       the University of Chicago for supplying the last two references.
*       S. Chandrasekhar corresponded with me during the preparation
*       of this software in the 1991-1992 timeframe. The intial version
*       of CHANDRA was written to compute supernova core characteristics
*       at Wichita State University in 1992, to support the author's
*       book SUPERNOVA VIGNETTES and a graduate seminar lecture on
*       supernovae.
*
*/
